@@ -69,6 +69,7 @@ static DB_output_t fake_out = {
 - (void)tearDown {
     streamer_set_playing_track (NULL);
     pl_item_unref (it);
+    ctx.it = NULL;
     ctx.plt = NULL;
 
     [super tearDown];
@@ -655,7 +656,7 @@ static DB_output_t fake_out = {
     XCTAssert(!strcmp (buffer, "stereo"), @"The actual output is: %s", buffer);
 }
 
-- (void)test_AndTrueArgs_ReturnsTrue {
+- (void)test_AndTrueArgsWithElse_ReturnsTrue {
     pl_replace_meta (it, "artist", "artist");
     pl_replace_meta (it, "album", "album");
 
@@ -665,6 +666,26 @@ static DB_output_t fake_out = {
     XCTAssert(!strcmp (buffer, "true"), @"The actual output is: %s", buffer);
 }
 
+- (void)test_AndTrueArgs_ReturnsTrue {
+    pl_replace_meta (it, "artist", "artist");
+    pl_replace_meta (it, "album", "album");
+
+    char *bc = tf_compile("$if($and(%artist%,%album%),true)");
+    tf_eval (&ctx, bc, buffer, 1000);
+    tf_free (bc);
+    XCTAssert(!strcmp (buffer, "true"), @"The actual output is: %s", buffer);
+}
+
+- (void)test_AndTrueArgs_ReturnsArtistAlbum {
+    pl_replace_meta (it, "artist", "artist");
+    pl_replace_meta (it, "album", "album");
+
+    char *bc = tf_compile("$if($and(%artist%,%album%),%artist% - %album%)");
+    tf_eval (&ctx, bc, buffer, 1000);
+    tf_free (bc);
+    XCTAssert(!strcmp (buffer, "artist - album"), @"The actual output is: %s", buffer);
+}
+
 - (void)test_AndTrueAndFalseArgs_ReturnsFalse {
     pl_replace_meta (it, "artist", "artist");
 
@@ -1271,7 +1292,18 @@ static DB_output_t fake_out = {
     XCTAssert(!strcmp (buffer, "1"), @"The actual output is: %s", buffer);
 }
 
-- (void)test_IsPlayingReturnValueTrue_CorrespondsToStringValue {
+- (void)test_isPlaying_StatePlayingAndStreamerTrackNotSameAsCtxTrack_ReturnsNone {
+    streamer_set_playing_track (it);
+    ctx.it = NULL;
+    plug_set_output (&fake_out);
+    fake_out_state_value = DDB_PLAYBACK_STATE_PLAYING;
+    char *bc = tf_compile("$if(%isplaying%,YES,NO) %isplaying%");
+    tf_eval (&ctx, bc, buffer, 1000);
+    tf_free (bc);
+    XCTAssert(!strcmp (buffer, "NO "), @"The actual output is: %s", buffer);
+}
+
+- (void)test_isPlaying_StatePlayingAndStreamerTrackSameAsCtxTrack_Returns1 {
     streamer_set_playing_track (it);
     plug_set_output (&fake_out);
     fake_out_state_value = DDB_PLAYBACK_STATE_PLAYING;
@@ -1281,7 +1313,7 @@ static DB_output_t fake_out = {
     XCTAssert(!strcmp (buffer, "YES 1"), @"The actual output is: %s", buffer);
 }
 
-- (void)test_IsPlayingReturnValueFalse_CorrespondsToStringValue {
+- (void)test_IsPlaying_StateStopped_ReturnsNone {
     plug_set_output (&fake_out);
     fake_out_state_value = DDB_PLAYBACK_STATE_STOPPED;
     char *bc = tf_compile("$if(%isplaying%,YES,NO) %isplaying%");
@@ -1290,7 +1322,39 @@ static DB_output_t fake_out = {
     XCTAssert(!strcmp (buffer, "NO "), @"The actual output is: %s", buffer);
 }
 
-- (void)test_IsPausedReturnValueTrue_CorrespondsToStringValue {
+- (void)test_isPaused_StatePlayingAndStreamerTrackNotSameAsCtxTrack_ReturnsNone {
+    streamer_set_playing_track (it);
+    ctx.it = NULL;
+    plug_set_output (&fake_out);
+    fake_out_state_value = DDB_PLAYBACK_STATE_PLAYING;
+    char *bc = tf_compile("$if(%ispaused%,YES,NO) %ispaused%");
+    tf_eval (&ctx, bc, buffer, 1000);
+    tf_free (bc);
+    XCTAssert(!strcmp (buffer, "NO "), @"The actual output is: %s", buffer);
+}
+
+- (void)test_isPaused_StatePlayingAndStreamerTrackSameAsCtxTrack_ReturnsNone {
+    streamer_set_playing_track (it);
+    plug_set_output (&fake_out);
+    fake_out_state_value = DDB_PLAYBACK_STATE_PLAYING;
+    char *bc = tf_compile("$if(%ispaused%,YES,NO) %ispaused%");
+    tf_eval (&ctx, bc, buffer, 1000);
+    tf_free (bc);
+    XCTAssert(!strcmp (buffer, "NO "), @"The actual output is: %s", buffer);
+}
+
+- (void)test_isPaused_StatePausedAndStreamerTrackNotSameAsCtxTrack_ReturnsNone {
+    streamer_set_playing_track (it);
+    ctx.it = NULL;
+    plug_set_output (&fake_out);
+    fake_out_state_value = DDB_PLAYBACK_STATE_PAUSED;
+    char *bc = tf_compile("$if(%ispaused%,YES,NO) %ispaused%");
+    tf_eval (&ctx, bc, buffer, 1000);
+    tf_free (bc);
+    XCTAssert(!strcmp (buffer, "NO "), @"The actual output is: %s", buffer);
+}
+
+- (void)test_isPaused_StatePausedAndStreamerTrackSameAsCtxTrack_Returns1 {
     streamer_set_playing_track (it);
     plug_set_output (&fake_out);
     fake_out_state_value = DDB_PLAYBACK_STATE_PAUSED;
@@ -1427,6 +1491,29 @@ static DB_output_t fake_out = {
     XCTAssert(!strcmp (buffer, "DeaDBeeF"), @"The actual output is: %s", buffer);
 }
 
+- (void)test_replace_emptyStringWithEmpty_shouldComplete {
+    XCTestExpectation *expectation = [[XCTestExpectation alloc] initWithDescription:@"Completed"];
+
+    char *bc = tf_compile("$replace(foobar,,)");
+
+    dispatch_async (dispatch_get_global_queue (DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^(void) {
+        tf_eval (&self->ctx, bc, self->buffer, 1000);
+        tf_free (bc);
+        [expectation fulfill];
+    });
+
+    [self waitForExpectations:@[expectation] timeout:1];
+}
+
+- (void)test_replace_emptyStringWithSomething_shouldProduceOriginalString {
+    char *bc = tf_compile("$replace(foobar,,bar)");
+
+    tf_eval (&self->ctx, bc, self->buffer, 1000);
+    tf_free (bc);
+
+    XCTAssert(!strcmp (buffer, "foobar"), @"The actual output is: %s", buffer);
+}
+
 - (void)test_FilenameExt_ReturnsFilenameWithExt {
     pl_replace_meta (it, ":URI", "/Users/User/MyFile.mod");
     char *bc = tf_compile("%filename_ext%");
@@ -2334,18 +2421,20 @@ static DB_output_t fake_out = {
     XCTAssert(!strcmp (buffer, "\t   hello  \t"), @"The actual output is: %s", buffer);
 }
 
+#pragma mark - Tint
+
 - (void)test_CalculateTintFromString_LeadingTint_Valid {
     char str[] = "\0331;1mhello";
 
-    int tintRanges[100];
+    tint_stop_t tintStops[100];
 
     char *output;
-    unsigned count = calculate_tint_ranges_from_string (str, tintRanges, 100, &output);
+    unsigned count = calculate_tint_stops_from_string (str, tintStops, 100, &output);
 
-    XCTAssertEqual (count, 2);
+    XCTAssertEqual (count, 1);
     XCTAssertTrue(!strcmp(output,"hello"));
-    XCTAssertEqual(tintRanges[0], 1);
-    XCTAssertEqual(tintRanges[1], 0);
+    XCTAssertEqual(tintStops[0].tint, 1);
+    XCTAssertEqual(tintStops[0].index, 0);
 
     free (output);
 }
@@ -2353,15 +2442,15 @@ static DB_output_t fake_out = {
 - (void)test_CalculateTintFromString_TrailingTint_Valid {
     char str[] = "hello\0331;1m";
 
-    int tintRanges[100];
+    tint_stop_t tintStops[100];
 
     char *output;
-    unsigned count = calculate_tint_ranges_from_string (str, tintRanges, 100, &output);
+    unsigned count = calculate_tint_stops_from_string (str, tintStops, 100, &output);
 
-    XCTAssertEqual (count, 2);
+    XCTAssertEqual (count, 1);
     XCTAssertTrue(!strcmp(output,"hello"));
-    XCTAssertEqual(tintRanges[0], 1);
-    XCTAssertEqual(tintRanges[1], 5);
+    XCTAssertEqual(tintStops[0].tint, 1);
+    XCTAssertEqual(tintStops[0].index, 5);
 
     free (output);
 }
@@ -2369,17 +2458,17 @@ static DB_output_t fake_out = {
 - (void)test_CalculateTintFromString_MultipleTintLeadingTrailing_Valid {
     char str[] = "\0331;-1mhello\0331;1m";
 
-    int tintRanges[100];
+    tint_stop_t tintStops[100];
 
     char *output;
-    unsigned count = calculate_tint_ranges_from_string (str, tintRanges, 100, &output);
+    unsigned count = calculate_tint_stops_from_string (str, tintStops, 100, &output);
 
-    XCTAssertEqual (count, 4);
+    XCTAssertEqual (count, 2);
     XCTAssertTrue(!strcmp(output,"hello"));
-    XCTAssertEqual(tintRanges[0], -1);
-    XCTAssertEqual(tintRanges[1], 0);
-    XCTAssertEqual(tintRanges[2], 0);
-    XCTAssertEqual(tintRanges[3], 5);
+    XCTAssertEqual(tintStops[0].tint, -1);
+    XCTAssertEqual(tintStops[0].index, 0);
+    XCTAssertEqual(tintStops[1].tint, 0);
+    XCTAssertEqual(tintStops[1].index, 5);
 
     free (output);
 }
@@ -2387,19 +2476,205 @@ static DB_output_t fake_out = {
 - (void)test_CalculateTintFromString_MultipleTintMiddle_Valid {
     char str[] = "Leading\0331;-5mMiddle\0331;5mTrailing";
 
-    int tintRanges[100];
+    tint_stop_t tintStops[100];
 
     char *output;
-    unsigned count = calculate_tint_ranges_from_string (str, tintRanges, 100, &output);
+    unsigned count = calculate_tint_stops_from_string (str, tintStops, 100, &output);
 
-    XCTAssertEqual (count, 4);
+    XCTAssertEqual (count, 2);
     XCTAssertTrue(!strcmp(output,"LeadingMiddleTrailing"));
-    XCTAssertEqual(tintRanges[0], -5);
-    XCTAssertEqual(tintRanges[1], 7);
-    XCTAssertEqual(tintRanges[2], 0);
-    XCTAssertEqual(tintRanges[3], 13);
+    XCTAssertEqual(tintStops[0].tint, -5);
+    XCTAssertEqual(tintStops[0].index, 7);
+    XCTAssertEqual(tintStops[1].tint, 0);
+    XCTAssertEqual(tintStops[1].index, 13);
 
     free (output);
 }
 
+#pragma mark - RGB
+
+- (void)test_CalculateRGBFromString_LeadingRGB_Valid {
+    char str[] = "\0332;20;30;40mhello";
+
+    tint_stop_t tintStops[100];
+
+    char *output;
+    unsigned count = calculate_tint_stops_from_string (str, tintStops, 100, &output);
+
+    XCTAssertEqual (count, 1);
+    XCTAssertTrue(!strcmp(output,"hello"));
+    XCTAssertEqual(tintStops[0].has_rgb, 1);
+    XCTAssertEqual(tintStops[0].r, 20);
+    XCTAssertEqual(tintStops[0].g, 30);
+    XCTAssertEqual(tintStops[0].b, 40);
+
+    free (output);
+}
+
+- (void)test_CalculateRGBFromString_Unterminated_Invalid {
+    char str[] = "\0332;20;30;40hello";
+
+    tint_stop_t tintStops[100];
+
+    char *output;
+    unsigned count = calculate_tint_stops_from_string (str, tintStops, 100, &output);
+
+    XCTAssertEqual (count, 0);
+    XCTAssertTrue(!strcmp(output,"\0332;20;30;40hello"));
+
+    free (output);
+}
+
+- (void)test_CalculateRGBFromString_Negative_Reset {
+    char str[] = "\0332;-20;30;-40mhello";
+
+    tint_stop_t tintStops[100];
+
+    char *output;
+    unsigned count = calculate_tint_stops_from_string (str, tintStops, 100, &output);
+
+    XCTAssertEqual (count, 1);
+    XCTAssertEqual(tintStops[0].has_rgb, 0);
+    XCTAssertTrue(!strcmp(output,"hello"));
+
+    free (output);
+}
+
+- (void)test_CalculateRGBFromString_LargerThan255_Clamped {
+    char str[] = "\0332;1000;30;2000mhello";
+
+    tint_stop_t tintStops[100];
+
+    char *output;
+    unsigned count = calculate_tint_stops_from_string (str, tintStops, 100, &output);
+
+    XCTAssertEqual (count, 1);
+    XCTAssertTrue(!strcmp(output,"hello"));
+    XCTAssertEqual(tintStops[0].has_rgb, 1);
+    XCTAssertEqual(tintStops[0].r, 255);
+    XCTAssertEqual(tintStops[0].g, 30);
+    XCTAssertEqual(tintStops[0].b, 255);
+
+    free (output);
+}
+
+- (void)test_CalculateRGBFromString_TrailingRGB_Valid {
+    char str[] = "hello\0332;20;30;40m";
+
+    tint_stop_t tintStops[100];
+
+    char *output;
+    unsigned count = calculate_tint_stops_from_string (str, tintStops, 100, &output);
+
+    XCTAssertEqual (count, 1);
+    XCTAssertTrue(!strcmp(output,"hello"));
+    XCTAssertEqual(tintStops[0].has_rgb, 1);
+    XCTAssertEqual(tintStops[0].r, 20);
+    XCTAssertEqual(tintStops[0].g, 30);
+    XCTAssertEqual(tintStops[0].b, 40);
+
+    free (output);
+}
+
+- (void)test_CalculateRGBFromString_MultipleRGBLeadingTrailing_Valid {
+    char str[] = "\0332;20;30;40mhello\0332;50;60;70m";
+
+    tint_stop_t tintStops[100];
+
+    char *output;
+    unsigned count = calculate_tint_stops_from_string (str, tintStops, 100, &output);
+
+    XCTAssertEqual (count, 2);
+    XCTAssertTrue(!strcmp(output,"hello"));
+    XCTAssertEqual(tintStops[0].has_rgb, 1);
+    XCTAssertEqual(tintStops[0].r, 20);
+    XCTAssertEqual(tintStops[0].g, 30);
+    XCTAssertEqual(tintStops[0].b, 40);
+    XCTAssertEqual(tintStops[1].has_rgb, 1);
+    XCTAssertEqual(tintStops[1].r, 50);
+    XCTAssertEqual(tintStops[1].g, 60);
+    XCTAssertEqual(tintStops[1].b, 70);
+
+    free (output);
+}
+
+- (void)test_CalculateRGBFromString_MultipleRGBMiddle_Valid {
+    char str[] = "Leading\0332;20;30;40mMiddle\0332;50;60;70mTrailing";
+
+    tint_stop_t tintStops[100];
+
+    char *output;
+    unsigned count = calculate_tint_stops_from_string (str, tintStops, 100, &output);
+
+    XCTAssertEqual (count, 2);
+    XCTAssertTrue(!strcmp(output,"LeadingMiddleTrailing"));
+    XCTAssertEqual(tintStops[0].has_rgb, 1);
+    XCTAssertEqual(tintStops[0].r, 20);
+    XCTAssertEqual(tintStops[0].g, 30);
+    XCTAssertEqual(tintStops[0].b, 40);
+    XCTAssertEqual(tintStops[1].has_rgb, 1);
+    XCTAssertEqual(tintStops[1].r, 50);
+    XCTAssertEqual(tintStops[1].g, 60);
+    XCTAssertEqual(tintStops[1].b, 70);
+
+    free (output);
+}
+
+#pragma mark - Tint + RGB
+
+- (void)test_CalculateRGBFromString_TintWithRGB_Valid {
+    char str[] = "\0331;-3m\0332;20;30;40mHello";
+
+    tint_stop_t tintStops[100];
+
+    char *output;
+    unsigned count = calculate_tint_stops_from_string (str, tintStops, 100, &output);
+
+    XCTAssertEqual (count, 2);
+    XCTAssertTrue(!strcmp(output,"Hello"));
+    XCTAssertEqual(tintStops[0].has_rgb, 0);
+    XCTAssertEqual(tintStops[0].tint, -3);
+    XCTAssertEqual(tintStops[1].has_rgb, 1);
+    XCTAssertEqual(tintStops[1].r, 20);
+    XCTAssertEqual(tintStops[1].g, 30);
+    XCTAssertEqual(tintStops[1].b, 40);
+
+    free (output);
+}
+
+- (void)test_year_empty_returnNothing {
+    char *bc = tf_compile("$year()");
+    tf_eval (&ctx, bc, buffer, 1000);
+    tf_free (bc);
+    XCTAssert(!strcmp (buffer, ""), @"The actual output is: %s", buffer);
+}
+
+- (void)test_year_text_returnNothing {
+    char *bc = tf_compile("$year(abcd)");
+    tf_eval (&ctx, bc, buffer, 1000);
+    tf_free (bc);
+    XCTAssert(!strcmp (buffer, ""), @"The actual output is: %s", buffer);
+}
+
+- (void)test_year_shorterThan4_returnNothing {
+    char *bc = tf_compile("$year(123)");
+    tf_eval (&ctx, bc, buffer, 1000);
+    tf_free (bc);
+    XCTAssert(!strcmp (buffer, ""), @"The actual output is: %s", buffer);
+}
+
+- (void)test_year_Exactly4_returnValue {
+    char *bc = tf_compile("$year(1234)");
+    tf_eval (&ctx, bc, buffer, 1000);
+    tf_free (bc);
+    XCTAssert(!strcmp (buffer, "1234"), @"The actual output is: %s", buffer);
+}
+
+- (void)test_year_LongerThan4WithText_returnYear {
+    char *bc = tf_compile("$year(9999text)");
+    tf_eval (&ctx, bc, buffer, 1000);
+    tf_free (bc);
+    XCTAssert(!strcmp (buffer, "9999"), @"The actual output is: %s", buffer);
+}
+
 @end
