@@ -24,8 +24,10 @@
 
   Alexey Yakovenko waker@users.sourceforge.net
 */
+#include <Block.h>
 #include <ctype.h>
 #include <dirent.h>
+#include <dispatch/dispatch.h>
 #include <dlfcn.h>
 #include <assert.h>
 #include <stdio.h>
@@ -66,6 +68,7 @@
 #ifdef __APPLE__
 #include "cocoautil.h"
 #endif
+#include "viz.h"
 
 DB_plugin_t main_plugin = {
     .type = DB_PLUGIN_MISC,
@@ -94,6 +97,7 @@ const char *lowprio_plugin_ids[] = {
 // internal plugin list
 typedef struct plugin_s {
     void *handle;
+    char *filepath;
     DB_plugin_t *plugin;
     struct plugin_s *next;
 } plugin_t;
@@ -133,6 +137,10 @@ static DB_playlist_t *g_playlist_plugins[MAX_PLAYLIST_PLUGINS+1];
 static uintptr_t background_jobs_mutex;
 static int num_background_jobs;
 
+static void
+_viz_spectrum_listen_stub (void *ctx, void (*callback)(void *ctx, const ddb_audio_data_t *data)) {
+}
+
 // deadbeef api
 static DB_functions_t deadbeef_api = {
     .vmajor = DB_API_VERSION_MAJOR,
@@ -413,10 +421,11 @@ static DB_functions_t deadbeef_api = {
     .plt_get_meta = (int (*) (ddb_playlist_t *handle, const char *key, char *val, int size))plt_get_meta,
     .pl_meta_exists = (int (*) (DB_playItem_t *it, const char *key))pl_meta_exists,
     // ******* new 1.5 APIs ********
-    .vis_waveform_listen = vis_waveform_listen,
-    .vis_waveform_unlisten = vis_waveform_unlisten,
-    .vis_spectrum_listen = vis_spectrum_listen,
-    .vis_spectrum_unlisten = vis_spectrum_unlisten,
+    .vis_waveform_listen = viz_waveform_listen,
+    .vis_waveform_unlisten = viz_waveform_unlisten,
+    .vis_spectrum_listen = _viz_spectrum_listen_stub,
+    .vis_spectrum_listen2 = viz_spectrum_listen,
+    .vis_spectrum_unlisten = viz_spectrum_unlisten,
     .audio_set_mute = audio_set_mute,
     .audio_is_mute = audio_is_mute,
     .background_job_increment = background_job_increment,
@@ -512,6 +521,16 @@ static DB_functions_t deadbeef_api = {
     .pl_find_meta_with_override = (const char *(*) (ddb_playItem_t *it, const char *key))pl_find_meta_with_override,
     .pl_get_meta_with_override = (int (*) (ddb_playItem_t *it, const char *key, char *val, size_t size))pl_get_meta_with_override,
     .pl_meta_exists_with_override = (int (*) (ddb_playItem_t *it, const char *key))pl_meta_exists_with_override,
+
+    .plt_item_set_selected = (void (*)(ddb_playlist_t *plt, ddb_playItem_t *it, int sel))pl_set_selected_in_playlist,
+    .plt_find_by_name = (ddb_playlist_t * (*) (const char *name))plt_find_by_name,
+    .plt_append = (ddb_playlist_t * (*) (const char *title))plt_append,
+
+    .plt_get_head_item = (ddb_playItem_t * (*) (ddb_playlist_t *p, int iter))plt_get_head_item,
+    .plt_get_tail_item = (ddb_playItem_t * (*) (ddb_playlist_t *p, int iter))plt_get_tail_item,
+
+    .plug_get_path_for_plugin_ptr = (const char* (*) (DB_plugin_t *plugin_ptr))plug_get_path_for_plugin_ptr,
+    .plt_insert_dir3 = (ddb_playItem_t *(*) (int visibility, uint32_t flags, ddb_playlist_t *plt, ddb_playItem_t *after, const char *dirname, int *pabort, int (*callback)(ddb_insert_file_result_t result, const char *fname, void *user_data), void *user_data))plt_insert_dir3,
 };
 
 DB_functions_t *deadbeef = &deadbeef_api;
@@ -556,6 +575,8 @@ plug_get_system_dir (int dir_id) {
         return dbpixmapdir;
     case DDB_SYS_DIR_CACHE:
         return dbcachedir;
+    case DDB_SYS_DIR_PLUGIN_RESOURCES:
+        return dbresourcedir;
     }
     return NULL;
 }
@@ -818,6 +839,7 @@ load_plugin (const char *plugdir, char *d_name, int l) {
         dlclose (handle);
         return -1;
     }
+    plugins_tail->filepath = strdup (fullname);
     return 0;
 }
 
@@ -825,7 +847,7 @@ static int
 load_gui_plugin (const char **plugdirs) {
 #if defined HAVE_COCOAUI || defined HAVE_XGUI
     return 0;
-#endif
+#else
 
     char conf_gui_plug[100];
     conf_get_str ("gui_plugin", "GTK2", conf_gui_plug, sizeof (conf_gui_plug));
@@ -860,6 +882,7 @@ load_gui_plugin (const char **plugdirs) {
         }
     }
     return -1;
+#endif
 }
 
 static int
@@ -914,7 +937,7 @@ load_plugin_dir (const char *plugdir, int gui_scan) {
                     while (*e && *e > 0x20) {
                         e++;
                     }
-                    if (l-sizeof (PLUGINEXT)+1 == e-p) {
+                    if (l-sizeof (PLUGINEXT)+1 == (uintptr_t)(e-p)) {
                         if (!strncmp (p, d_name, e-p)) {
                             p = NULL;
                             break;
@@ -954,10 +977,10 @@ load_plugin_dir (const char *plugdir, int gui_scan) {
                             break;
                         }
                         // add to list of unique names
-                        size_t i;
-                        for (i = 0; g_gui_names[i] && strcmp(g_gui_names[i], nm); i++);
-                        if (!g_gui_names[i]) {
-                            g_gui_names[i] = strdup (nm);
+                        size_t idx;
+                        for (idx = 0; g_gui_names[idx] && strcmp(g_gui_names[idx], nm); idx++);
+                        if (!g_gui_names[idx]) {
+                            g_gui_names[idx] = strdup (nm);
                             g_gui_names[++g_num_gui_names] = NULL;
                             trace ("added %s gui plugin\n", nm);
                         }
@@ -1228,7 +1251,9 @@ plug_connect_all (void) {
     for (plug = plugins; plug;) {
         if (plug->plugin->connect) {
             if (plug->plugin->connect () < 0) {
-                trace ("plugin %s failed to connect to dependencies, deactivated.\n", plug->plugin->name);
+                // NOTE: This message is confusing, and looks like a serious error.
+                // Potentially, it should be logged at some additional verbose log level.
+                // trace ("plugin %s failed to connect to dependencies, deactivated.\n", plug->plugin->name);
 
                 if (plug->plugin->disconnect) {
                     plug->plugin->disconnect ();
@@ -1275,14 +1300,26 @@ plug_disconnect_all (void) {
     }
 }
 
-void
-plug_unload_all (void) {
-    action_set_playlist (NULL);
-    trace ("plug_unload_all\n");
-    plugin_t *p;
-    for (p = plugins; p; p = p->next) {
-        if (p->plugin->stop) {
-            trace ("stopping %s...\n", p->plugin->name);
+static void
+_plug_unload_stop_complete (void);
+
+static int _async_stop_count = 0;
+static void (^_async_stop_completion_block)(void);
+
+static void
+_handle_async_stop (void) {
+    _async_stop_count -= 1;
+    if (_async_stop_count == 0) {
+        _plug_unload_stop_complete();
+    }
+}
+
+static void
+_plug_unload_stop_complete (void) {
+    // Stop the normal plugins with synchronous stop
+    for (plugin_t *p = plugins; p; p = p->next) {
+        if (p->plugin->stop && !(p->plugin->flags & DDB_PLUGIN_FLAG_ASYNC_STOP)) {
+            trace ("Stopping %s...\n", p->plugin->name);
             fflush (stderr);
 #if HAVE_COCOAUI
             if (p->plugin->type == DB_PLUGIN_GUI) {
@@ -1292,12 +1329,13 @@ plug_unload_all (void) {
             p->plugin->stop ();
         }
     }
-    trace ("stopped all plugins\n");
+
     while (plugins) {
         plugin_t *next = plugins->next;
         if (plugins->handle) {
             dlclose (plugins->handle);
         }
+        free (plugins->filepath);
         free (plugins);
         plugins = next;
     }
@@ -1317,11 +1355,35 @@ plug_unload_all (void) {
     output_plugin = NULL;
     memset (g_playlist_plugins, 0, sizeof (g_playlist_plugins));
 
-    trace ("all plugins had been unloaded\n");
+    trace ("All plugins had been unloaded\n");
     if (background_jobs_mutex) {
         mutex_free (background_jobs_mutex);
         background_jobs_mutex = 0;
     }
+    _async_stop_completion_block();
+    Block_release(_async_stop_completion_block);
+    _async_stop_completion_block = NULL;
+}
+
+void
+plug_unload_all (void(^completion_block)(void)) {
+    _async_stop_completion_block = Block_copy(completion_block);
+    action_set_playlist (NULL);
+    trace ("plug_unload_all\n");
+    trace ("Waiting for async plugins to finish...\n");
+    _async_stop_count = 1;
+    for (plugin_t *p = plugins; p; p = p->next) {
+        if ((p->plugin->flags & DDB_PLUGIN_FLAG_ASYNC_STOP) && p->plugin->command) {
+            _async_stop_count += 1;
+            if (0 > p->plugin->command(DDB_COMMAND_PLUGIN_ASYNC_STOP, ^{
+                trace ("Stopped %s...\n", p->plugin->name);
+                _handle_async_stop();
+            })) {
+                _async_stop_count -= 1;
+            }
+        }
+    }
+    _handle_async_stop();
 }
 
 void
@@ -1457,10 +1519,10 @@ plug_get_decoder_for_id (const char *id) {
     if (!id) {
         return NULL;
     }
-    DB_decoder_t **plugins = plug_get_decoder_list ();
-    for (int c = 0; plugins[c]; c++) {
-        if (!strcmp (id, plugins[c]->plugin.id)) {
-            return plugins[c];
+    DB_decoder_t **decoder_plugins = plug_get_decoder_list ();
+    for (int c = 0; decoder_plugins[c]; c++) {
+        if (!strcmp (id, decoder_plugins[c]->plugin.id)) {
+            return decoder_plugins[c];
         }
     }
     return NULL;
@@ -1468,10 +1530,21 @@ plug_get_decoder_for_id (const char *id) {
 
 DB_plugin_t *
 plug_get_for_id (const char *id) {
-    DB_plugin_t **plugins = plug_get_list ();
-    for (int c = 0; plugins[c]; c++) {
-        if (plugins[c]->id && !strcmp (id, plugins[c]->id)) {
-            return plugins[c];
+    DB_plugin_t **all_plugins = plug_get_list ();
+    for (int c = 0; all_plugins[c]; c++) {
+        if (all_plugins[c]->id && !strcmp (id, all_plugins[c]->id)) {
+            return all_plugins[c];
+        }
+    }
+    return NULL;
+}
+
+const char *
+plug_get_path_for_plugin_ptr (DB_plugin_t *plugin_ptr) {
+    plugin_t *p;
+    for (p = plugins; p; p = p->next) {
+        if (p->plugin == plugin_ptr) {
+            return p->filepath;
         }
     }
     return NULL;
@@ -1556,7 +1629,7 @@ is_relative_path_win32 (const char *path_or_url) {
         if (isalpha(path_or_url[0]) && path_or_url[1] == ':' && (path_or_url[2] == '\\' || path_or_url[2] == '/')) {
             return 0;
         }
-        else if (path_or_url[0] == '\\' && path_or_url[1] == '\\') {
+        else if ((path_or_url[0] == '\\' && path_or_url[1] == '\\') || (path_or_url[0] == '/' && path_or_url[1] == '/')) {
             return 0;
         }
     }
