@@ -36,7 +36,7 @@ int
 plug_load_all (void);
 
 void
-plug_unload_all (void);
+plug_unload_all (void(^completion_block)(void));
 
 void
 plug_connect_all (void);
@@ -175,4 +175,7 @@ plug_get_api (void);
 int
 plug_init_plugin (DB_plugin_t* (*loadfunc)(DB_functions_t *), void *handle);
 
+const char *
+plug_get_path_for_plugin_ptr (DB_plugin_t *plugin_ptr);
+
 #endif // __PLUGINS_H
