@@ -1,7 +1,6 @@
 /*
-    Album Art plugin for DeaDBeeF
-    Copyright (C) 2009-2011 Viktor Semykin <thesame.ml@gmail.com>
-    Copyright (C) 2009-2013 Alexey Yakovenko <waker@users.sourceforge.net>
+    DeaDBeeF -- the music player
+    Copyright (C) 2009-2021 Alexey Yakovenko and other contributors
 
     This software is provided 'as-is', without any express or implied
     warranty.  In no event will the authors be held liable for any damages
@@ -25,6 +24,7 @@
 #ifdef HAVE_CONFIG_H
     #include "../../config.h"
 #endif
+#include <dispatch/dispatch.h>
 #include <stdlib.h>
 #include <string.h>
 #include <libgen.h>
@@ -40,39 +40,63 @@
 #define trace(...)
 
 extern DB_functions_t *deadbeef;
+extern dispatch_queue_t sync_queue;
 
-static DB_FILE *http_request;
-static uintptr_t http_mutex;
+#define MAX_REQUESTS FETCH_CONCURRENT_LIMIT
+static DB_FILE *_requests[MAX_REQUESTS]; // all current open requests
 
 static DB_FILE *
-new_http_request (const char *url)
-{
+new_http_request (const char *url) {
     errno = 0;
 
-    if (!http_mutex) {
-        http_mutex = deadbeef->mutex_create_nonrecursive ();
-        if (!http_mutex) {
-            return NULL;
+    DB_FILE *http_request = deadbeef->fopen (url);
+
+    __block int registered = 0;
+
+    dispatch_sync(sync_queue, ^{
+        for (int i = 0; i < MAX_REQUESTS; i++) {
+            if (_requests[i] == NULL) {
+                _requests[i] = http_request;
+                registered = 1;
+                break;
+            }
         }
+    });
+
+    if (!registered) {
+        deadbeef->fclose (http_request);
+        trace ("Too many concurrent HTTP requests");
+        return NULL;
     }
 
-    deadbeef->mutex_lock (http_mutex);
-    http_request = deadbeef->fopen (url);
-    deadbeef->mutex_unlock (http_mutex);
     return http_request;
 }
 
-static void
-close_http_request (DB_FILE *request)
-{
-    deadbeef->mutex_lock (http_mutex);
-    deadbeef->fclose (request);
-    http_request = NULL;
-    deadbeef->mutex_unlock (http_mutex);
+// Make sure to call on sync queue
+void
+artwork_abort_all_http_requests (void) {
+    for (int i = 0; i < MAX_REQUESTS; i++) {
+        if (_requests[i] != NULL) {
+            deadbeef->fabort (_requests[i]);
+        }
+    }
 }
 
-size_t artwork_http_request (const char *url, char *buffer, const size_t buffer_size)
-{
+static void
+close_http_request (DB_FILE *request) {
+    dispatch_sync(sync_queue, ^{
+        for (int i = 0; i < MAX_REQUESTS; i++) {
+            if (_requests[i] == request) {
+                _requests[i] = NULL;
+                break;
+            }
+        }
+    });
+    deadbeef->fclose (request);
+}
+
+size_t
+artwork_http_request (const char *url, char *buffer, const size_t buffer_size) {
     DB_FILE *request = new_http_request (url);
     if (!request) {
         return 0;
@@ -86,21 +110,8 @@ size_t artwork_http_request (const char *url, char *buffer, const size_t buffer_
     return size;
 }
 
-void artwork_abort_http_request (void)
-{
-    if (http_mutex) {
-        deadbeef->mutex_lock (http_mutex);
-        if (http_request) {
-            deadbeef->fabort (http_request);
-        }
-        http_request = NULL;
-        deadbeef->mutex_unlock (http_mutex);
-    }
-}
-
 static int
-check_dir (const char *path)
-{
+check_dir (const char *path) {
     struct stat stat_struct;
     if (!stat (path, &stat_struct)) {
         return S_ISDIR (stat_struct.st_mode);
@@ -142,6 +153,12 @@ copy_file (const char *in, const char *out) {
 
     char tmp_out[PATH_MAX];
     snprintf (tmp_out, PATH_MAX, "%s.part", out);
+
+    struct stat stat_struct;
+    if (!stat (tmp_out, &stat_struct) && S_ISREG (stat_struct.st_mode) && stat_struct.st_size > 0) {
+        return 0;
+    }
+
     FILE *fout = fopen (tmp_out, "w+b");
     if (!fout) {
         trace ("artwork: failed to open file %s for writing\n", tmp_out);
@@ -179,11 +196,14 @@ copy_file (const char *in, const char *out) {
     }
 
     unlink (tmp_out);
+    if (!err && file_bytes == 0) {
+        return -1;
+    }
     return err;
 }
 
-int write_file (const char *out, const char *data, const size_t data_length)
-{
+int
+write_file (const char *out, const char *data, const size_t data_length) {
     if (!ensure_dir (out)) {
         return -1;
     }
