@@ -33,28 +33,30 @@
 extern DB_functions_t *deadbeef;
 static NSString *default_format = @"[%tracknumber%. ][%artist% - ]%title%";
 
-@interface ConverterWindowController () <ScriptableSelectDelegate,ScriptableItemDelegate> {
-    ddb_converter_t *_converter_plugin;
+@interface ConverterWindowController () <ScriptableSelectDelegate,ScriptableItemDelegate,NSControlTextEditingDelegate>
 
-    NSCondition *_overwritePromptCondition;
+@property (nonatomic) ddb_converter_t *converter_plugin;
 
-    // conversion context
-    DB_playItem_t **_convert_items;
-    ddb_playlist_t *_convert_playlist;
-    int _convert_items_count;
-    NSString *_outfolder;
-    NSString *_outfile;
-    int _preserve_folder_structure;
-    int _write_to_source_folder;
-    int _output_bps;
-    int _output_is_float;
-    int _overwrite_action;
-    int _cancelled;
-    NSInteger _overwritePromptResult;
-    BOOL _working;
-    NSInteger _bypassSameFormatState;
-    NSInteger _retagAfterCopyState;
-}
+@property (nonatomic) NSCondition *overwritePromptCondition;
+
+// conversion context
+@property (nonatomic) NSString *outfolder;
+@property (nonatomic) NSString *outfile;
+@property (nonatomic) int preserve_folder_structure;
+@property (nonatomic) int write_to_source_folder;
+@property (nonatomic) int output_bps;
+@property (nonatomic) int output_is_float;
+@property (nonatomic) int overwrite_action;
+@property (nonatomic) int cancelled;
+@property (nonatomic) NSInteger overwritePromptResult;
+@property (nonatomic) BOOL working;
+@property (nonatomic) NSInteger bypassSameFormatState;
+@property (nonatomic) NSInteger retagAfterCopyState;
+
+
+@property (nonatomic) ddb_playItem_t **convert_items;
+@property (nonatomic) ddb_playlist_t *convert_playlist;
+@property (nonatomic) NSInteger convert_items_count;
 
 @property (nonatomic,unsafe_unretained) ddb_dsp_preset_t *dsp_preset;
 @property (nonatomic,unsafe_unretained) ddb_encoder_preset_t *encoder_preset;
@@ -67,6 +69,37 @@ static NSString *default_format = @"[%tracknumber%. ][%artist% - ]%title%";
 @property (nonatomic) ScriptableTableDataSource *encoderPresetsDataSource;
 @property (weak) IBOutlet NSView *encoderPresetSelectorContainer;
 
+@property (unsafe_unretained) IBOutlet NSTextField *outputFolder;
+@property (unsafe_unretained) IBOutlet NSButton *writeToSourceFolder;
+@property (unsafe_unretained) IBOutlet NSButton *preserveFolderStructure;
+@property (unsafe_unretained) IBOutlet NSButton *bypassSameFormat;
+@property (weak) IBOutlet NSButton *retagAfterCopy;
+@property (unsafe_unretained) IBOutlet NSTextField *outputFileName;
+@property (unsafe_unretained) IBOutlet NSArrayController *filenamePreviewController;
+@property (unsafe_unretained) IBOutlet NSPopUpButton *outputFormat;
+@property (unsafe_unretained) IBOutlet NSPopUpButton *fileExistsAction;
+- (IBAction)cancelAction:(id)sender;
+- (IBAction)okAction:(id)sender;
+
+- (IBAction)openOutputFolderAction:(id)sender;
+
+- (IBAction)writeToSourceFolderChanged:(id)sender;
+- (IBAction)preserveFolderStructureChanged:(id)sender;
+- (IBAction)bypassSameFormatChanged:(id)sender;
+- (IBAction)retagAfterCopyChanged:(id)sender;
+- (IBAction)overwritePromptChanged:(id)sender;
+- (IBAction)outputFormatChanged:(id)sender;
+
+@property (strong) IBOutlet NSPanel *progressPanel;
+@property (unsafe_unretained) IBOutlet NSTextField *progressText;
+@property (unsafe_unretained) IBOutlet NSTextField *progressOutText;
+@property (unsafe_unretained) IBOutlet NSTextField *progressNumeric;
+
+@property (unsafe_unretained) IBOutlet NSProgressIndicator *progressBar;
+- (IBAction)progressCancelAction:(id)sender;
+
+
+
 @end
 
 static NSMutableArray *g_converterControllers;
@@ -77,7 +110,7 @@ static NSMutableArray *g_converterControllers;
     [super windowDidLoad];
 
     // Implement this method to handle any initialization after your window controller's window has been loaded from its nib file.
-    _converter_plugin = (ddb_converter_t *)deadbeef->plug_get_for_id ("converter");
+    self.converter_plugin = (ddb_converter_t *)deadbeef->plug_get_for_id ("converter");
 
     [self initializeWidgets];
     self.window.delegate = self;
@@ -130,17 +163,17 @@ static NSMutableArray *g_converterControllers;
         out_folder = getenv("HOME");
     }
 
-    _outputFolder.stringValue = [NSString stringWithUTF8String:out_folder];
-    _outputFileName.stringValue = [NSString stringWithUTF8String:deadbeef->conf_get_str_fast ("converter.output_file", "")];
-    _preserveFolderStructure.state = deadbeef->conf_get_int ("converter.preserve_folder_structure", 0) ? NSOnState : NSOffState;
-    _bypassSameFormat.state = deadbeef->conf_get_int ("converter.bypass_same_format", 0);
-    _retagAfterCopy.state = deadbeef->conf_get_int ("converter.retag_after_copy", 0);
-    _retagAfterCopy.enabled = _bypassSameFormat.state == NSOnState;
+    self.outputFolder.stringValue = [NSString stringWithUTF8String:out_folder];
+    self.outputFileName.stringValue = [NSString stringWithUTF8String:deadbeef->conf_get_str_fast ("converter.output_file", "")];
+    self.preserveFolderStructure.state = deadbeef->conf_get_int ("converter.preserve_folder_structure", 0) ? NSControlStateValueOn : NSControlStateValueOff;
+    self.bypassSameFormat.state = deadbeef->conf_get_int ("converter.bypass_same_format", 0);
+    self.retagAfterCopy.state = deadbeef->conf_get_int ("converter.retag_after_copy", 0);
+    self.retagAfterCopy.enabled = self.bypassSameFormat.state == NSControlStateValueOn;
 
     int write_to_source_folder = deadbeef->conf_get_int ("converter.write_to_source_folder", 0);
-    _writeToSourceFolder.state = write_to_source_folder ? NSOnState : NSOffState;
-    _outputFolder.enabled = !write_to_source_folder;
-    _preserveFolderStructure.enabled = !write_to_source_folder;
+    self.writeToSourceFolder.state = write_to_source_folder ? NSControlStateValueOn : NSControlStateValueOff;
+    self.outputFolder.enabled = !write_to_source_folder;
+    self.preserveFolderStructure.enabled = !write_to_source_folder;
 
     [_fileExistsAction selectItemAtIndex:deadbeef->conf_get_int ("converter.overwrite_action", 0)];
     deadbeef->conf_unlock ();
@@ -151,12 +184,12 @@ static NSMutableArray *g_converterControllers;
 
 - (void)controlTextDidChange:(NSNotification *)notification {
     NSTextField *textField = [notification object];
-    if (textField == _outputFolder) {
+    if (textField == self.outputFolder) {
         [self updateFilenamesPreview];
         deadbeef->conf_set_str ("converter.output_folder", [[_outputFolder stringValue] UTF8String]);
         deadbeef->conf_save ();
     }
-    else if (textField == _outputFileName) {
+    else if (textField == self.outputFileName) {
         [self updateFilenamesPreview];
         deadbeef->conf_set_str ("converter.output_file", [[_outputFileName stringValue] UTF8String]);
         deadbeef->conf_save ();
@@ -165,19 +198,19 @@ static NSMutableArray *g_converterControllers;
 
 - (IBAction)preserveFolderStructureChanged:(id)sender {
     [self updateFilenamesPreview];
-    deadbeef->conf_set_int ("converter.preserve_folder_structure", _preserveFolderStructure.state == NSOnState);
+    deadbeef->conf_set_int ("converter.preserve_folder_structure", self.preserveFolderStructure.state == NSControlStateValueOn);
     deadbeef->conf_save ();
 }
 
 - (IBAction)bypassSameFormatChanged:(id)sender {
-    deadbeef->conf_set_int ("converter.bypass_same_format", _bypassSameFormat.state == NSOnState);
+    deadbeef->conf_set_int ("converter.bypass_same_format", self.bypassSameFormat.state == NSControlStateValueOn);
     deadbeef->conf_save ();
 
-    _retagAfterCopy.enabled = _bypassSameFormat.state == NSOnState;
+    self.retagAfterCopy.enabled = self.bypassSameFormat.state == NSControlStateValueOn;
 }
 
 - (IBAction)retagAfterCopyChanged:(id)sender {
-    deadbeef->conf_set_int ("converter.retag_after_copy", _retagAfterCopy.state == NSOnState);
+    deadbeef->conf_set_int ("converter.retag_after_copy", self.retagAfterCopy.state == NSControlStateValueOn);
     deadbeef->conf_save ();
 }
 
@@ -190,7 +223,7 @@ static NSMutableArray *g_converterControllers;
         return;
     }
     scriptableItem_t *preset = scriptableItemChildAtIndex(scriptableEncoderRoot(), (unsigned int)selectedEncoderPreset);
-    ddb_encoder_preset_t *encoder_preset = _converter_plugin->encoder_preset_alloc();
+    ddb_encoder_preset_t *encoder_preset = self.converter_plugin->encoder_preset_alloc();
     scriptableEncoderPresetToConverterEncoderPreset(preset, encoder_preset);
 
     NSString *outfile = [_outputFileName stringValue];
@@ -199,27 +232,27 @@ static NSMutableArray *g_converterControllers;
         outfile = default_format;
     }
 
-    for (int n = 0; n < _convert_items_count; n++) {
-        DB_playItem_t *it = _convert_items[n];
+    for (int n = 0; n < self.convert_items_count; n++) {
+        DB_playItem_t *it = self.convert_items[n];
         if (it) {
             char outpath[PATH_MAX];
 
-            _converter_plugin->get_output_path2 (it, _convert_playlist, [[_outputFolder stringValue] UTF8String], [outfile UTF8String], encoder_preset, _preserveFolderStructure.state == NSOnState, "", _writeToSourceFolder.state == NSOnState, outpath, sizeof (outpath));
+            self.converter_plugin->get_output_path2 (it, self.convert_playlist, [[_outputFolder stringValue] UTF8String], [outfile UTF8String], encoder_preset, self.preserveFolderStructure.state == NSControlStateValueOn, "", self.writeToSourceFolder.state == NSControlStateValueOn, outpath, sizeof (outpath));
 
             [convert_items_preview addObject:[NSString stringWithUTF8String:outpath]];
         }
     }
 
-    _filenamePreviewController.content = convert_items_preview;
+    self.filenamePreviewController.content = convert_items_preview;
 }
 
 - (IBAction)writeToSourceFolderChanged:(NSButton *)sender {
     [self updateFilenamesPreview];
-    int active = sender.state == NSOnState;
+    int active = sender.state == NSControlStateValueOn;
     deadbeef->conf_set_int ("converter.write_to_source_folder", active);
     deadbeef->conf_save ();
-    _outputFolder.enabled = !active;
-    _preserveFolderStructure.enabled = !active;
+    self.outputFolder.enabled = !active;
+    self.preserveFolderStructure.enabled = !active;
 }
 
 
@@ -234,26 +267,26 @@ static NSMutableArray *g_converterControllers;
 }
 
 - (IBAction)progressCancelAction:(id)sender {
-    _cancelled = YES;
+    self.cancelled = YES;
 }
 
 - (void)reset {
     if (_convert_items) {
-        for (int n = 0; n < _convert_items_count; n++) {
+        for (NSInteger n = 0; n < self.convert_items_count; n++) {
             deadbeef->pl_item_unref (_convert_items[n]);
         }
         free (_convert_items);
-        _convert_items = NULL;
+        self.convert_items = NULL;
     }
-    _convert_items_count = 0;
+    self.convert_items_count = 0;
 
     if (_convert_playlist) {
         deadbeef->plt_unref (_convert_playlist);
-        _convert_playlist = NULL;
+        self.convert_playlist = NULL;
     }
 
-    _outfolder = nil;
-    _outfile = nil;
+    self.outfolder = nil;
+    self.outfile = nil;
 
     if (self.encoder_preset) {
         self.encoder_preset = NULL;
@@ -263,8 +296,8 @@ static NSMutableArray *g_converterControllers;
     }
 }
 
-- (void)run:(int)ctx {
-    if (_converter_plugin) {
+- (void)runWithTracks:(ddb_playItem_t **)tracks count:(NSInteger)count playlist:(ddb_playlist_t *)plt {
+    if (self.converter_plugin) {
         [self initializeWidgets];
     }
     [self showWindow:self];
@@ -273,73 +306,21 @@ static NSMutableArray *g_converterControllers;
     // reinit everything
     [self reset];
 
-    // store list of tracks
-    deadbeef->pl_lock ();
-    switch (ctx) {
-        case DDB_ACTION_CTX_MAIN:
-        case DDB_ACTION_CTX_SELECTION:
-        {
-            // copy list
-            ddb_playlist_t *plt = deadbeef->plt_get_curr ();
-            if (plt) {
-                _convert_playlist = plt;
-                _convert_items_count = deadbeef->plt_getselcount (plt);
-                if (0 < _convert_items_count) {
-                    _convert_items = malloc (sizeof (DB_playItem_t *) * _convert_items_count);
-                    if (_convert_items) {
-                        int n = 0;
-                        DB_playItem_t *it = deadbeef->pl_get_first (PL_MAIN);
-                        while (it) {
-                            if (deadbeef->pl_is_selected (it)) {
-                                assert (n < _convert_items_count);
-                                deadbeef->pl_item_ref (it);
-                                _convert_items[n++] = it;
-                            }
-                            DB_playItem_t *next = deadbeef->pl_get_next (it, PL_MAIN);
-                            deadbeef->pl_item_unref (it);
-                            it = next;
-                        }
-                    }
-                }
-            }
-            break;
-        }
-        case DDB_ACTION_CTX_PLAYLIST:
-        {
-            // copy list
-            ddb_playlist_t *plt = deadbeef->plt_get_curr ();
-            if (plt) {
-                _convert_playlist = plt;
-                _convert_items_count = deadbeef->plt_get_item_count (plt, PL_MAIN);
-                if (0 < _convert_items_count) {
-                    _convert_items = malloc (sizeof (DB_playItem_t *) * _convert_items_count);
-                    if (_convert_items) {
-                        int n = 0;
-                        DB_playItem_t *it = deadbeef->pl_get_first (PL_MAIN);
-                        while (it) {
-                            _convert_items[n++] = it;
-                            it = deadbeef->pl_get_next (it, PL_MAIN);
-                        }
-                    }
-                }
-            }
-            break;
-        }
-        case DDB_ACTION_CTX_NOWPLAYING:
-        {
-            DB_playItem_t *it = deadbeef->streamer_get_playing_track ();
-            if (it) {
-                _convert_playlist = deadbeef->pl_get_playlist (it);
-                _convert_items_count = 1;
-                _convert_items = malloc (sizeof (DB_playItem_t *) * _convert_items_count);
-                if (_convert_items) {
-                    _convert_items[1] = it;
-                }
-            }
-        }
-            break;
+    if (plt) {
+        deadbeef->plt_ref (plt);
+        self.convert_playlist = plt;
     }
-    deadbeef->pl_unlock ();
+
+    // store list of tracks
+    self.convert_items_count = count;
+    self.convert_items = calloc (count, sizeof (ddb_playItem_t *));
+
+    for (NSInteger i = 0; i < count; i++) {
+        ddb_playItem_t *it = tracks[i];
+        deadbeef->pl_item_ref (it);
+        self.convert_items[i] = it;
+    }
+
     [self updateFilenamesPreview];
 }
 
@@ -355,7 +336,7 @@ static NSMutableArray *g_converterControllers;
 }
 
 - (void)windowWillClose:(NSNotification *)notification {
-    if (!_working) {
+    if (!self.working) {
         [self converterFinished:self withResult:0];
     }
 }
@@ -372,59 +353,59 @@ static NSMutableArray *g_converterControllers;
     [panel beginSheetModalForWindow:self.window completionHandler:^(NSInteger result){
         if (result == NSModalResponseOK) {
             NSURL * url = [panel URL];
-            _outputFolder.stringValue =  [url path];
+            self.outputFolder.stringValue =  [url path];
         }
     }];
 }
 
 - (void)setEncoder_preset:(ddb_encoder_preset_t *)encoder_preset {
     if (_encoder_preset) {
-        _converter_plugin->encoder_preset_free (_encoder_preset);
+        self.converter_plugin->encoder_preset_free (_encoder_preset);
     }
     _encoder_preset = encoder_preset;
 }
 
 - (void)setDsp_preset:(ddb_dsp_preset_t *)dsp_preset {
     if (_dsp_preset) {
-        _converter_plugin->dsp_preset_free (_dsp_preset);
+        self.converter_plugin->dsp_preset_free (_dsp_preset);
     }
     _dsp_preset = dsp_preset;
 }
 
 - (IBAction)okAction:(id)sender {
-    _outfolder = [_outputFolder stringValue];
+    self.outfolder = [_outputFolder stringValue];
 
-    _outfile = [_outputFileName stringValue];
+    self.outfile = [_outputFileName stringValue];
 
-    if ([_outfile isEqual:@""]) {
-        _outfile = default_format;
+    if ([self.outfile isEqual:@""]) {
+        self.outfile = default_format;
     }
 
-    _preserve_folder_structure = _preserveFolderStructure.state == NSOnState;
+    self.preserve_folder_structure = self.preserveFolderStructure.state == NSControlStateValueOn;
 
-    _write_to_source_folder = _writeToSourceFolder.state == NSOnState;
+    self.write_to_source_folder = self.writeToSourceFolder.state == NSControlStateValueOn;
 
-    _overwrite_action = (int)[_fileExistsAction indexOfSelectedItem];
+    self.overwrite_action = (int)[_fileExistsAction indexOfSelectedItem];
 
     int selected_format = (int)[_outputFormat indexOfSelectedItem];
     switch (selected_format) {
         case 1 ... 4:
-            _output_bps = selected_format * 8;
-            _output_is_float = 0;
+            self.output_bps = selected_format * 8;
+            self.output_is_float = 0;
             break;
         case 5:
-            _output_bps = 32;
-            _output_is_float = 1;
+            self.output_bps = 32;
+            self.output_is_float = 1;
             break;
         default:
-            _output_bps = -1; // same as input, or encoder default
+            self.output_bps = -1; // same as input, or encoder default
             break;
     }
 
     NSInteger selectedEncoderPreset = self.encoderSelectViewController.indexOfSelectedItem;
     if (selectedEncoderPreset != -1) {
         scriptableItem_t *preset = scriptableItemChildAtIndex(scriptableEncoderRoot(), (unsigned int)selectedEncoderPreset);
-        self.encoder_preset = _converter_plugin->encoder_preset_alloc();
+        self.encoder_preset = self.converter_plugin->encoder_preset_alloc();
         scriptableEncoderPresetToConverterEncoderPreset(preset, self.encoder_preset);
     }
 
@@ -446,26 +427,26 @@ static NSMutableArray *g_converterControllers;
         scriptableItem_t *preset = scriptableItemChildAtIndex(scriptableDspRoot(), (unsigned int)selectedDspPreset);
         ddb_dsp_context_t *chain = scriptableDspConfigToDspChain(preset);
         if (chain) {
-            self.dsp_preset = _converter_plugin->dsp_preset_alloc ();
+            self.dsp_preset = self.converter_plugin->dsp_preset_alloc ();
             self.dsp_preset->chain = chain;
         }
     }
 
-    _cancelled = NO;
+    self.cancelled = NO;
     self.window.isVisible = NO;
 
-    _progressText.stringValue = @"";
-    _progressOutText.stringValue = @"";
-    _progressNumeric.stringValue = @"";
-    _progressBar.minValue = 0;
-    _progressBar.maxValue = _convert_items_count-1;
-    _progressBar.doubleValue = 0;
-    _progressPanel.isVisible = YES;
+    self.progressText.stringValue = @"";
+    self.progressOutText.stringValue = @"";
+    self.progressNumeric.stringValue = @"";
+    self.progressBar.minValue = 0;
+    self.progressBar.maxValue = self.convert_items_count-1;
+    self.progressBar.doubleValue = 0;
+    self.progressPanel.isVisible = YES;
 
-    _working = YES;
+    self.working = YES;
 
-    _bypassSameFormatState = _bypassSameFormat.state;
-    _retagAfterCopyState = _retagAfterCopy.state;
+    self.bypassSameFormatState = self.bypassSameFormat.state;
+    self.retagAfterCopyState = self.retagAfterCopy.state;
 
     dispatch_queue_t aQueue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
     dispatch_async(aQueue, ^{
@@ -479,7 +460,7 @@ static NSMutableArray *g_converterControllers;
     char root[2000] = "";
     size_t rootlen = 0;
     // prepare for preserving folder struct
-    if (_preserve_folder_structure && _convert_items_count >= 1) {
+    if (self.preserve_folder_structure && self.convert_items_count >= 1) {
         // start with the 1st track path
         deadbeef->pl_get_meta (_convert_items[0], ":URI", root, sizeof (root));
         char *sep = strrchr (root, '/');
@@ -488,7 +469,7 @@ static NSMutableArray *g_converterControllers;
         }
         // reduce
         rootlen = strlen (root);
-        for (int n = 1; n < _convert_items_count; n++) {
+        for (int n = 1; n < self.convert_items_count; n++) {
             deadbeef->pl_lock ();
             const char *path = deadbeef->pl_find_meta (_convert_items[n], ":URI");
             if (strncmp (path, root, rootlen)) {
@@ -513,27 +494,27 @@ static NSMutableArray *g_converterControllers;
     }
 
     ddb_converter_settings_t settings = {
-        .output_bps = _output_bps,
-        .output_is_float = _output_is_float,
+        .output_bps = self.output_bps,
+        .output_is_float = self.output_is_float,
         .encoder_preset = self.encoder_preset,
         .dsp_preset = self.dsp_preset,
-        .bypass_conversion_on_same_format = (_bypassSameFormatState == NSOnState),
-        .rewrite_tags_after_copy = (_retagAfterCopyState == NSOnState),
+        .bypass_conversion_on_same_format = (self.bypassSameFormatState == NSControlStateValueOn),
+        .rewrite_tags_after_copy = (self.retagAfterCopyState == NSControlStateValueOn),
     };
 
-    for (int n = 0; n < _convert_items_count; n++) {
+    for (int n = 0; n < self.convert_items_count; n++) {
         deadbeef->pl_lock ();
         NSString *text = [NSString stringWithUTF8String:deadbeef->pl_find_meta (_convert_items[n], ":URI")];
         deadbeef->pl_unlock ();
         char outpath[PATH_MAX];
-        _converter_plugin->get_output_path2 (_convert_items[n], _convert_playlist, [_outfolder UTF8String], [_outfile UTF8String], _encoder_preset, _preserve_folder_structure, root, _write_to_source_folder, outpath, sizeof (outpath));
+        self.converter_plugin->get_output_path2 (_convert_items[n], self.convert_playlist, [self.outfolder UTF8String], [self.outfile UTF8String], self.encoder_preset, self.preserve_folder_structure, root, self.write_to_source_folder, outpath, sizeof (outpath));
         NSString *nsoutpath = [NSString stringWithUTF8String:outpath];
 
         dispatch_async(dispatch_get_main_queue(), ^{
-            _progressBar.doubleValue = n;
-            _progressText.stringValue = text;
-            _progressOutText.stringValue = nsoutpath;
-            _progressNumeric.stringValue = [NSString stringWithFormat:@"%d/%d", n+1, _convert_items_count];
+            self.progressBar.doubleValue = n;
+            self.progressText.stringValue = text;
+            self.progressOutText.stringValue = nsoutpath;
+            self.progressNumeric.stringValue = [NSString stringWithFormat:@"%d/%d", n+1, (int)self.convert_items_count];
         });
 
         int skip = 0;
@@ -549,7 +530,7 @@ static NSMutableArray *g_converterControllers;
             if (paths_match) {
                 fprintf (stderr, "converter: destination file is the same as source file, skipping\n");
             }
-            else if (_overwrite_action == 2) {
+            else if (self.overwrite_action == 2) {
                 unlink (outpath);
                 skip = 0;
             }
@@ -560,30 +541,30 @@ static NSMutableArray *g_converterControllers;
                     skip = 0;
                 }
                 else if (result == NSAlertThirdButtonReturn) {
-                    _cancelled = YES;
+                    self.cancelled = YES;
                 }
             }
         }
 
         if (!skip) {
-            _converter_plugin->convert2 (&settings, _convert_items[n], outpath, &_cancelled);
+            self.converter_plugin->convert2 (&settings, self.convert_items[n], outpath, &_cancelled);
         }
-        if (_cancelled) {
+        if (self.cancelled) {
             break;
         }
     }
     dispatch_async(dispatch_get_main_queue(), ^{
-        [_progressPanel close];
+        [self.progressPanel close];
         [self converterFinished:self withResult:1];
     });
 
     deadbeef->background_job_decrement ();
 
-    _working = NO;
+    self.working = NO;
 }
 
 - (NSInteger)overwritePrompt:(NSString *)path {
-    _overwritePromptCondition = [NSCondition new];
+    self.overwritePromptCondition = [NSCondition new];
     dispatch_sync(dispatch_get_main_queue(), ^{
         NSAlert *alert = [NSAlert new];
         [alert addButtonWithTitle:@"No"];
@@ -594,28 +575,28 @@ static NSMutableArray *g_converterControllers;
         alert.alertStyle = NSAlertStyleCritical;
 
         [alert beginSheetModalForWindow:_progressPanel completionHandler:^(NSModalResponse returnCode) {
-            [_overwritePromptCondition lock];
-            _overwritePromptResult = returnCode;
-            [_overwritePromptCondition signal];
-            [_overwritePromptCondition unlock];
+            [self.overwritePromptCondition lock];
+            self.overwritePromptResult = returnCode;
+            [self.overwritePromptCondition signal];
+            [self.overwritePromptCondition unlock];
         }];
     });
 
-    [_overwritePromptCondition lock];
-    [_overwritePromptCondition wait];
-    [_overwritePromptCondition unlock];
-    _overwritePromptCondition = nil;
-    return _overwritePromptResult;
+    [self.overwritePromptCondition lock];
+    [self.overwritePromptCondition wait];
+    [self.overwritePromptCondition unlock];
+    self.overwritePromptCondition = nil;
+    return self.overwritePromptResult;
 }
 
-+ (void)runConverter:(int)ctx {
++ (void)runConverterWithTracks:(ddb_playItem_t **)tracks count:(NSInteger)count playlist:(ddb_playlist_t *)plt {
     ConverterWindowController *conv = [[ConverterWindowController alloc] initWithWindowNibName:@"Converter"];
 
     if (!g_converterControllers) {
         g_converterControllers = [NSMutableArray new];
     }
     [g_converterControllers addObject:conv];
-    [conv run:DDB_ACTION_CTX_SELECTION];
+    [conv runWithTracks:tracks count:count playlist:plt];
 }
 
 + (void)cleanup {
