@@ -21,7 +21,6 @@
     3. This notice may not be removed or altered from any source distribution.
 */
 
-#import "DdbWidget.h"
 #import "PlaylistHeaderView.h"
 #import "PlaylistContentView.h"
 
@@ -30,5 +29,6 @@
 @property (nonatomic,readonly) PlaylistHeaderView *headerView;
 @property (nonatomic,readonly) PlaylistContentView *contentView;
 @property (weak,nonatomic) id<DdbListviewDelegate> delegate;
+@property (weak,nonatomic) id<DdbListviewDataModelProtocol> dataModel;
 
 @end
