@@ -24,18 +24,21 @@
 #ifndef deadbeef_DdbShared_h
 #define deadbeef_DdbShared_h
 
+#import <Foundation/Foundation.h>
+
 extern NSString * const ddbPlaylistItemsUTIType;
+extern NSString * const ddbMedialibItemUTIType;
 
 int
 cocoaui_add_new_playlist (void);
 
-void
-cocoaui_playlist_set_curr (int playlist);
-
 NSString *
 conf_get_nsstr (const char *key, const char *def);
 
 void
 conf_set_nsstr (const char *key, NSString *value);
 
+NSString *
+plt_get_title_wrapper (int plt);
+
 #endif
