@@ -32,47 +32,19 @@
 #include <libavutil/avstring.h>
 
 
-#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(54, 6, 0)
-#define avformat_find_stream_info(ctx,data) av_find_stream_info(ctx)
-#define avcodec_open2(ctx,codec,data) avcodec_open(ctx,codec)
-#endif
-
-#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(55, 16, 101)
-#define avformat_close_input(ctx) {av_close_input_file(*(ctx)); *(ctx) = NULL;}
-#endif
-
-#if LIBAVCODEC_VERSION_INT > AV_VERSION_INT(55, 28, 0)
-#    define avcodec_free_frame(frame) av_frame_free(frame)
-#elif LIBAVCODEC_VERSION_INT > AV_VERSION_INT(54, 59, 100)
-     // has avcodec_free_frame
-#elif LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(53, 40, 0)
-#    define avcodec_free_frame(frame) av_freep(frame) // newest -- av_freep
-#endif
+#define avcodec_free_frame(frame) av_frame_free(frame)
 
 #if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(57, 8, 0)
 #define av_packet_unref(packet) av_free_packet(packet)
 #endif
 
-#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(52, 64, 0)
-#define AVMEDIA_TYPE_AUDIO CODEC_TYPE_AUDIO
-#endif
-
-
-#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(55, 28, 1)
-#define av_frame_alloc() avcodec_alloc_frame()
-#endif
-
-#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(53, 2, 0)
-#define avformat_open_input(ctx, uri, fmt, options) av_open_input_file(ctx, uri, NULL, fmt, options)
-#endif
-
 //#define trace(...) { fprintf(stderr, __VA_ARGS__); }
 #define trace(fmt,...)
 
 #define min(x,y) ((x)<(y)?(x):(y))
 #define max(x,y) ((x)>(y)?(x):(y))
 
-static DB_decoder_t plugin;
+static ddb_decoder2_t plugin;
 static DB_functions_t *deadbeef;
 
 #define DEFAULT_EXTS "aa3;oma;ac3;vqf;amr;tak;dsf;dff;wma;3gp;mp4;m4a"
@@ -87,25 +59,14 @@ static DB_functions_t *deadbeef;
 
 static char * exts[EXT_MAX+1] = {NULL};
 
-enum {
-    FT_ALAC = 0,
-    FT_WMA = 1,
-    FT_ATRAC3 = 2,
-    FT_VQF = 3,
-    FT_AC3 = 4,
-    FT_AMR = 5,
-    FT_UNKNOWN = 5
-};
-
 typedef struct {
     DB_fileinfo_t info;
     AVCodec *codec;
-    AVCodecContext *ctx;
-    AVFormatContext *fctx;
+    AVCodecContext *codec_context;
+    int need_to_free_codec_context;
+    AVFormatContext *format_context;
     AVPacket pkt;
-#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(53, 40, 0)
     AVFrame *frame;
-#endif
     int stream_id;
 
     int left_in_packet;
@@ -122,19 +83,19 @@ typedef struct {
 
 static DB_fileinfo_t *
 ffmpeg_open (uint32_t hints) {
-    ffmpeg_info_t *info = calloc (sizeof (ffmpeg_info_t), 1);
+    ffmpeg_info_t *info = calloc (1, sizeof (ffmpeg_info_t));
     return &info->info;
 }
 
 // ensure that the buffer can contain entire frame of frame_size bytes per channel
 static int
 ensure_buffer (ffmpeg_info_t *info, int frame_size) {
-    if (!info->buffer || info->buffer_size < frame_size * info->ctx->channels) {
+    if (!info->buffer || info->buffer_size < frame_size * info->codec_context->channels) {
         if (info->buffer) {
             free (info->buffer);
             info->buffer = NULL;
         }
-        info->buffer_size = frame_size*info->ctx->channels;
+        info->buffer_size = frame_size*info->codec_context->channels;
         info->left_in_buffer = 0;
         int err = posix_memalign ((void **)&info->buffer, 16, info->buffer_size);
         if (err) {
@@ -145,6 +106,40 @@ ensure_buffer (ffmpeg_info_t *info, int frame_size) {
     return 0;
 }
 
+static int
+_get_audio_codec_from_stream(AVFormatContext *format_context, int stream_index, ffmpeg_info_t *info) {
+#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(57, 33, 0)
+    if (format_context->streams[stream_index]->codecpar->codec_type != AVMEDIA_TYPE_AUDIO) {
+        return 0;
+    }
+    AVCodec *codec = avcodec_find_decoder(format_context->streams[stream_index]->codecpar->codec_id);
+    if (codec == NULL) {
+        return 0;
+    }
+    info->codec = codec;
+    info->stream_id = stream_index;
+    info->codec_context = avcodec_alloc_context3 (info->codec);
+    info->need_to_free_codec_context = 1;
+    avcodec_parameters_to_context(info->codec_context, format_context->streams[stream_index]->codecpar);
+#else
+    if (format_context->streams[stream_index]->codec->codec_type != AVMEDIA_TYPE_AUDIO) {
+        return 0;
+    }
+    AVCodecContext *ctx = format_context->streams[stream_index]->codec;
+    if (ctx == NULL) {
+        return 0;
+    }
+    AVCodec *codec = avcodec_find_decoder (ctx->codec_id);
+    if (codec == NULL) {
+        return 0;
+    }
+    info->codec_context = ctx;
+    info->codec = codec;
+    info->stream_id = stream_index;
+#endif
+    return 1;
+}
+
 static int
 ffmpeg_init (DB_fileinfo_t *_info, DB_playItem_t *it) {
     // Don't allow playing network streams.
@@ -174,9 +169,9 @@ ffmpeg_init (DB_fileinfo_t *_info, DB_playItem_t *it) {
     // open file
     trace ("\033[0;31mffmpeg av_open_input_file\033[37;0m\n");
 
-    info->fctx = avformat_alloc_context ();
+    info->format_context = avformat_alloc_context ();
 
-    if ((ret = avformat_open_input(&info->fctx, uri, NULL, NULL)) < 0) {
+    if ((ret = avformat_open_input(&info->format_context, uri, NULL, NULL)) < 0) {
         trace ("\033[0;31minfo->fctx is %p, ret %d/%s\033[0;31m\n", info->fctx, ret, strerror(-ret));
         return -1;
     }
@@ -184,18 +179,12 @@ ffmpeg_init (DB_fileinfo_t *_info, DB_playItem_t *it) {
 
     trace ("\033[0;31mffmpeg avformat_find_stream_info\033[37;0m\n");
     info->stream_id = -1;
-    info->fctx->max_analyze_duration = AV_TIME_BASE;
-    avformat_find_stream_info(info->fctx, NULL);
-    for (i = 0; i < info->fctx->nb_streams; i++)
-    {
-        info->ctx = info->fctx->streams[i]->codec;
-        if (info->ctx->codec_type == AVMEDIA_TYPE_AUDIO)
-        {
-            info->codec = avcodec_find_decoder (info->ctx->codec_id);
-            if (info->codec != NULL) {
-                info->stream_id = i;
-                break;
-            }
+    info->format_context->max_analyze_duration = AV_TIME_BASE;
+    avformat_find_stream_info(info->format_context, NULL);
+    AVFormatContext *fctx = info->format_context;
+    for (i = 0; i < info->format_context->nb_streams; i++) {
+        if (_get_audio_codec_from_stream (fctx, i, info)) {
+            break;
         }
     }
 
@@ -207,21 +196,21 @@ ffmpeg_init (DB_fileinfo_t *_info, DB_playItem_t *it) {
     trace ("ffmpeg can decode %s\n", deadbeef->pl_find_meta (it, ":URI"));
     trace ("ffmpeg: codec=%s, stream=%d\n", info->codec->name, i);
 
-    if (avcodec_open2 (info->ctx, info->codec, NULL) < 0) {
+    if (avcodec_open2 (info->codec_context, info->codec, NULL) < 0) {
         trace ("ffmpeg: avcodec_open2 failed\n");
         return -1;
     }
 
     deadbeef->pl_replace_meta (it, "!FILETYPE", info->codec->name);
 
-    int bps = av_get_bytes_per_sample (info->ctx->sample_fmt)*8;
-    int samplerate = info->ctx->sample_rate;
+    int bps = av_get_bytes_per_sample (info->codec_context->sample_fmt)*8;
+    int samplerate = info->codec_context->sample_rate;
 
-    if (bps <= 0 || info->ctx->channels <= 0 || samplerate <= 0) {
+    if (bps <= 0 || info->codec_context->channels <= 0 || samplerate <= 0) {
         return -1;
     }
 
-    int64_t totalsamples = info->fctx->duration * samplerate / AV_TIME_BASE;
+    int64_t totalsamples = info->format_context->duration * samplerate / AV_TIME_BASE;
     info->left_in_packet = 0;
     info->left_in_buffer = 0;
 
@@ -231,12 +220,12 @@ ffmpeg_init (DB_fileinfo_t *_info, DB_playItem_t *it) {
     info->frame = av_frame_alloc();
 
     // fill in mandatory plugin fields
-    _info->plugin = &plugin;
+    _info->plugin = &plugin.decoder;
     _info->readpos = 0;
     _info->fmt.bps = bps;
-    _info->fmt.channels = info->ctx->channels;
+    _info->fmt.channels = info->codec_context->channels;
     _info->fmt.samplerate = samplerate;
-    if (info->ctx->sample_fmt == AV_SAMPLE_FMT_FLT || info->ctx->sample_fmt == AV_SAMPLE_FMT_FLTP) {
+    if (info->codec_context->sample_fmt == AV_SAMPLE_FMT_FLT || info->codec_context->sample_fmt == AV_SAMPLE_FMT_FLTP) {
         _info->fmt.is_float = 1;
     }
 
@@ -253,7 +242,7 @@ ffmpeg_init (DB_fileinfo_t *_info, DB_playItem_t *it) {
     if (endsample > 0) {
         info->startsample = deadbeef->pl_item_get_startsample (it);
         info->endsample = endsample;
-        plugin.seek_sample (_info, 0);
+        plugin.seek_sample64 (_info, 0);
     }
     else {
         info->startsample = 0;
@@ -262,28 +251,37 @@ ffmpeg_init (DB_fileinfo_t *_info, DB_playItem_t *it) {
     return 0;
 }
 
+static void
+_free_info_data(ffmpeg_info_t *info) {
+    if (info->frame) {
+        avcodec_free_frame(&info->frame);
+    }
+    if (info->buffer) {
+        free (info->buffer);
+    }
+    // free everything allocated in _init and _read
+    if (info->have_packet) {
+        av_packet_unref (&info->pkt);
+    }
+    if (info->codec_context) {
+        avcodec_close (info->codec_context);
+
+        // The ctx is owned by AVFormatContext in legacy mode
+        if (info->need_to_free_codec_context) {
+            avcodec_free_context (&info->codec_context);
+        }
+    }
+    if (info->format_context) {
+        avformat_close_input (&info->format_context);
+    }
+}
+
 static void
 ffmpeg_free (DB_fileinfo_t *_info) {
     trace ("ffmpeg: free\n");
     ffmpeg_info_t *info = (ffmpeg_info_t*)_info;
     if (info) {
-        if (info->frame) {
-            avcodec_free_frame(&info->frame);
-        }
-        if (info->buffer) {
-            free (info->buffer);
-        }
-        // free everything allocated in _init and _read
-        if (info->have_packet) {
-            av_packet_unref (&info->pkt);
-        }
-        if (info->ctx) {
-            avcodec_close (info->ctx);
-        }
-        if (info->fctx) {
-            avformat_close_input (&info->fctx);
-        }
-
+        _free_info_data(info);
         free (info);
     }
 }
@@ -293,10 +291,10 @@ ffmpeg_read (DB_fileinfo_t *_info, char *bytes, int size) {
     trace ("ffmpeg_read_int16 %d\n", size);
     ffmpeg_info_t *info = (ffmpeg_info_t*)_info;
 
-    _info->fmt.channels = info->ctx->channels;
-    _info->fmt.samplerate = info->ctx->sample_rate;
-    _info->fmt.bps = av_get_bytes_per_sample (info->ctx->sample_fmt) * 8;
-    _info->fmt.is_float = (info->ctx->sample_fmt == AV_SAMPLE_FMT_FLT || info->ctx->sample_fmt == AV_SAMPLE_FMT_FLTP);
+    _info->fmt.channels = info->codec_context->channels;
+    _info->fmt.samplerate = info->codec_context->sample_rate;
+    _info->fmt.bps = av_get_bytes_per_sample (info->codec_context->sample_fmt) * 8;
+    _info->fmt.is_float = (info->codec_context->sample_fmt == AV_SAMPLE_FMT_FLT || info->codec_context->sample_fmt == AV_SAMPLE_FMT_FLTP);
 
     int samplesize = _info->fmt.channels * _info->fmt.bps / 8;
 
@@ -330,36 +328,50 @@ ffmpeg_read (DB_fileinfo_t *_info, char *bytes, int size) {
 
         while (info->left_in_packet > 0 && size > 0) {
             int out_size = info->buffer_size;
-            int len;
+            int len = 0;
             //trace ("in: out_size=%d(%d), size=%d\n", out_size, AVCODEC_MAX_AUDIO_FRAME_SIZE, size);
 
-#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(53, 40, 0)
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(55, 28, 0)
+            int ret = avcodec_send_packet (info->codec_context, &info->pkt);
+            if (ret < 0) {
+                break;
+            }
+            ret = avcodec_receive_frame (info->codec_context,info->frame);
+            if (ret < 0 && ret != AVERROR(EAGAIN) && ret != AVERROR_EOF) {
+                break;
+            }
+            else {
+                len = info->pkt.size;
+            }
+#else
             int got_frame = 0;
             len = avcodec_decode_audio4(info->ctx, info->frame, &got_frame, &info->pkt);
+#endif
+
             if (len > 0) {
                 if (ensure_buffer (info, info->frame->nb_samples * (_info->fmt.bps >> 3))) {
                     return -1;
                 }
-                if (av_sample_fmt_is_planar(info->ctx->sample_fmt)) {
+                if (av_sample_fmt_is_planar(info->codec_context->sample_fmt)) {
                     out_size = 0;
-                    for (int c = 0; c < info->ctx->channels; c++) {
+                    for (int c = 0; c < info->codec_context->channels; c++) {
                         for (int i = 0; i < info->frame->nb_samples; i++) {
                             if (_info->fmt.bps == 8) {
-                                info->buffer[i*info->ctx->channels+c] = ((int8_t *)info->frame->extended_data[c])[i];
+                                info->buffer[i*info->codec_context->channels+c] = ((int8_t *)info->frame->extended_data[c])[i];
                                 out_size++;
                             }
                             else if (_info->fmt.bps == 16) {
                                 int16_t outsample = ((int16_t *)info->frame->extended_data[c])[i];
-                                ((int16_t*)info->buffer)[i*info->ctx->channels+c] = outsample;
+                                ((int16_t*)info->buffer)[i*info->codec_context->channels+c] = outsample;
                                 out_size += 2;
                             }
                             else if (_info->fmt.bps == 24) {
-                                memcpy (&info->buffer[(i*info->ctx->channels+c)*3], &((int8_t*)info->frame->extended_data[c])[i*3], 3);
+                                memcpy (&info->buffer[(i*info->codec_context->channels+c)*3], &((int8_t*)info->frame->extended_data[c])[i*3], 3);
                                 out_size += 3;
                             }
                             else if (_info->fmt.bps == 32) {
                                 int32_t sample = ((int32_t *)info->frame->extended_data[c])[i];
-                                ((int32_t*)info->buffer)[i*info->ctx->channels+c] = sample;
+                                ((int32_t*)info->buffer)[i*info->codec_context->channels+c] = sample;
                                 out_size += 4;
                             }
                         }
@@ -371,16 +383,6 @@ ffmpeg_read (DB_fileinfo_t *_info, char *bytes, int size) {
                 }
             }
 
-#else
-            if (ensure_buffer (info, 16384)) { // FIXME: how to get the packet size in old ffmpeg?
-                return -1;
-            }
-#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(52,25,0)
-            len = avcodec_decode_audio3 (info->ctx, (int16_t *)info->buffer, &out_size, &info->pkt);
-#else
-            len = avcodec_decode_audio2 (info->ctx, (int16_t *)info->buffer, &out_size, info->pkt.data, info->pkt.size);
-#endif
-#endif
             trace ("out: out_size=%d, len=%d\n", out_size, len);
             if (len <= 0) {
                 break;
@@ -402,7 +404,7 @@ ffmpeg_read (DB_fileinfo_t *_info, char *bytes, int size) {
         int errcount = 0;
         for (;;) {
             int ret;
-            if ((ret = av_read_frame (info->fctx, &info->pkt)) < 0) {
+            if ((ret = av_read_frame (info->format_context, &info->pkt)) < 0) {
                 trace ("ffmpeg: error %d\n", ret);
                 if (ret == AVERROR_EOF || ret == -1) {
                     ret = -1;
@@ -436,7 +438,7 @@ ffmpeg_read (DB_fileinfo_t *_info, char *bytes, int size) {
             info->left_in_packet = info->pkt.size;
 
             if (info->pkt.duration > 0) {
-                AVRational *time_base = &info->fctx->streams[info->stream_id]->time_base;
+                AVRational *time_base = &info->format_context->streams[info->stream_id]->time_base;
                 float sec = (float)info->pkt.duration * time_base->num / time_base->den;
                 int bitrate = info->pkt.size * 8 / sec;
                 if (bitrate > 0) {
@@ -458,7 +460,7 @@ ffmpeg_read (DB_fileinfo_t *_info, char *bytes, int size) {
 }
 
 static int
-ffmpeg_seek_sample (DB_fileinfo_t *_info, int sample) {
+ffmpeg_seek_sample64 (DB_fileinfo_t *_info, int64_t sample) {
     ffmpeg_info_t *info = (ffmpeg_info_t*)_info;
     // seek to specified sample (frame)
     // return 0 on success
@@ -468,11 +470,11 @@ ffmpeg_seek_sample (DB_fileinfo_t *_info, int sample) {
         info->have_packet = 0;
     }
     sample += info->startsample;
-    int64_t tm = (int64_t)sample/ _info->fmt.samplerate * AV_TIME_BASE;
+    int64_t tm = sample / _info->fmt.samplerate * AV_TIME_BASE;
     trace ("ffmpeg: seek to sample: %d, t: %d\n", sample, (int)tm);
     info->left_in_packet = 0;
     info->left_in_buffer = 0;
-    if (av_seek_frame (info->fctx, -1, tm, AVSEEK_FLAG_ANY) < 0) {
+    if (av_seek_frame (info->format_context, -1, tm, AVSEEK_FLAG_ANY) < 0) {
         trace ("ffmpeg: seek error\n");
         return -1;
     }
@@ -483,9 +485,14 @@ ffmpeg_seek_sample (DB_fileinfo_t *_info, int sample) {
     return 0;
 }
 
+static int
+ffmpeg_seek_sample (DB_fileinfo_t *_info, int sample) {
+    return ffmpeg_seek_sample64(_info, sample);
+}
+
 static int
 ffmpeg_seek (DB_fileinfo_t *_info, float time) {
-    return ffmpeg_seek_sample (_info, time * _info->fmt.samplerate);
+    return ffmpeg_seek_sample64 (_info, time * _info->fmt.samplerate);
 }
 
 static const char *map[] = {
@@ -542,49 +549,6 @@ ff_add_track_meta (DB_playItem_t *it, const char *track) {
 
 static int
 ffmpeg_read_metadata_internal (DB_playItem_t *it, AVFormatContext *fctx) {
-#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(52,43,0)
-    if (!strlen (fctx->title)) {
-        // title is empty, this call will set track title to filename without extension
-        deadbeef->pl_add_meta (it, "title", NULL);
-    }
-    else {
-        deadbeef->pl_add_meta (it, "title", fctx->title);
-    }
-    deadbeef->pl_add_meta (it, "artist", fctx->author);
-    deadbeef->pl_add_meta (it, "album", fctx->album);
-    deadbeef->pl_add_meta (it, "copyright", fctx->copyright);
-    deadbeef->pl_add_meta (it, "comment", fctx->comment);
-    deadbeef->pl_add_meta (it, "genre", fctx->genre);
-
-    char tmp[10];
-    snprintf (tmp, sizeof (tmp), "%d", fctx->year);
-    deadbeef->pl_add_meta (it, "year", tmp);
-    snprintf (tmp, sizeof (tmp), "%d", fctx->track);
-    deadbeef->pl_add_meta (it, "track", tmp);
-#else
-#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(54,23,0)
-    AVMetadata *md = fctx->metadata;
-
-    for (int m = 0; map[m]; m += 2) {
-        AVMetadataTag *tag = NULL;
-        do {
-            tag = av_metadata_get (md, map[m], tag, AV_METADATA_DONT_STRDUP_KEY | AV_METADATA_DONT_STRDUP_VAL);
-            if (tag) {
-                if (!strcmp (map[m+1], "disc")) {
-                    ff_add_disc_meta (it, tag->value);
-                }
-                else if (!strcmp (map[m+1], "track")) {
-                    ff_add_track_meta (it, tag->value);
-                }
-                else {
-                    deadbeef->pl_append_meta (it, map[m+1], tag->value);
-                }
-            }
-
-        } while (tag);
-    }
-#else
-    // ffmpeg-0.11 new metadata format
     AVDictionaryEntry *t = NULL;
     int m;
     for (int i = 0; i < fctx->nb_streams + 1; i++) {
@@ -629,8 +593,6 @@ ffmpeg_read_metadata_internal (DB_playItem_t *it, AVFormatContext *fctx) {
             }
         }
     }
-#endif
-#endif
     return 0;
 }
 
@@ -655,95 +617,71 @@ ffmpeg_insert (ddb_playlist_t *plt, DB_playItem_t *after, const char *fname) {
     // return track pointer on success
     // return NULL on failure
 
-    AVCodec *codec = NULL;
-    AVCodecContext *ctx = NULL;
-    AVFormatContext *fctx = NULL;
+    ffmpeg_info_t info = {0};
+
     int ret;
     char *uri = NULL;
     int i;
 
     // construct uri
-#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(54, 6, 0)
     uri = strdupa (fname);
-#else
-    uri = alloca (l + sizeof (FF_PROTOCOL_NAME) + 1);
-    memcpy (uri, FF_PROTOCOL_NAME, sizeof (FF_PROTOCOL_NAME)-1);
-    memcpy (uri + sizeof (FF_PROTOCOL_NAME)-1, ":", 1);
-    memcpy (uri + sizeof (FF_PROTOCOL_NAME), fname, l);
-    uri[sizeof (FF_PROTOCOL_NAME) + l] = 0;
-#endif
     trace ("ffmpeg: uri: %s\n", uri);
 
     // open file
-#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(54, 6, 0)
-    fctx = avformat_alloc_context ();
-    fctx->max_analyze_duration = AV_TIME_BASE;
-    if ((ret = avformat_open_input(&fctx, uri, NULL, NULL)) < 0) {
-#else
-    if ((ret = av_open_input_file(&fctx, uri, NULL, 0, NULL)) < 0) {
-#endif
+    info.format_context = avformat_alloc_context ();
+    info.format_context->max_analyze_duration = AV_TIME_BASE;
+    if ((ret = avformat_open_input(&info.format_context, uri, NULL, NULL)) < 0) {
         print_error (uri, ret);
-        return NULL;
+        goto error;
     }
 
-    trace ("fctx is %p, ret %d/%s\n", fctx, ret, strerror(-ret));
-    ret = avformat_find_stream_info(fctx, NULL);
+    trace ("fctx is %p, ret %d/%s\n", info.format_context, ret, strerror(-ret));
+    ret = avformat_find_stream_info(info.format_context, NULL);
     if (ret < 0) {
         trace ("avformat_find_stream_info ret: %d/%s\n", ret, strerror(-ret));
     }
     trace ("nb_streams=%x\n", nb_streams);
-    for (i = 0; i < fctx->nb_streams; i++)
+
+    for (i = 0; i < info.format_context->nb_streams; i++)
     {
-        if (!fctx->streams[i]) {
+        if (!info.format_context->streams[i]) {
             continue;
         }
-        ctx = fctx->streams[i]->codec;
-        if (ctx->codec_type ==
-#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(52, 64, 0)
-            AVMEDIA_TYPE_AUDIO)
-#else
-            CODEC_TYPE_AUDIO)
-#endif
-        {
-            codec = avcodec_find_decoder(ctx->codec_id);
-            if (codec != NULL) {
-                break;
-            }
+        if (_get_audio_codec_from_stream(info.format_context, i, &info)) {
+            break;
         }
     }
-//    AVStream *stream = fctx->streams[i];
 
-    if (codec == NULL)
+    if (info.codec == NULL)
     {
         trace ("ffmpeg can't decode %s\n", fname);
-        avformat_close_input (&fctx);
-        return NULL;
+        goto error;
     }
     trace ("ffmpeg can decode %s\n", fname);
     trace ("ffmpeg: codec=%s, stream=%d\n", codec->name, i);
 
-    if (avcodec_open2 (ctx, codec, NULL) < 0) {
+    int avcodec_open2_ret = avcodec_open2 (info.codec_context, info.codec, NULL);
+    if (avcodec_open2_ret < 0) {
         trace ("ffmpeg: avcodec_open2 failed\n");
-        avformat_close_input (&fctx);
-        return NULL;
+        goto error;
     }
 
-    int bps = av_get_bytes_per_sample (ctx->sample_fmt) * 8;
-    int samplerate = ctx->sample_rate;
-    float duration = fctx->duration / (float)AV_TIME_BASE;
+    int bps = av_get_bytes_per_sample (info.codec_context->sample_fmt) * 8;
+    int samplerate = info.codec_context->sample_rate;
+    float duration = info.format_context->duration / (float)AV_TIME_BASE;
 //    float duration = stream->duration * stream->time_base.num / (float)stream->time_base.den;
     trace ("ffmpeg: bits per sample is %d\n", bps);
     trace ("ffmpeg: samplerate is %d\n", samplerate);
     trace ("ffmpeg: duration is %f\n", duration);
 
-    if (bps <= 0 || ctx->channels <= 0 || samplerate <= 0) {
-        return NULL;
+    if (bps <= 0 || info.codec_context->channels <= 0 || samplerate <= 0) {
+        goto error;
     }
 
-    int64_t totalsamples = fctx->duration * samplerate / AV_TIME_BASE;
+    int64_t totalsamples = info.format_context->duration * samplerate / AV_TIME_BASE;
 
-    DB_playItem_t *it = deadbeef->pl_item_alloc_init (fname, plugin.plugin.id);
-    deadbeef->pl_replace_meta (it, ":FILETYPE", codec->name);
+    DB_playItem_t *it = deadbeef->pl_item_alloc_init (fname, plugin.decoder.plugin.id);
+    deadbeef->pl_replace_meta (it, ":FILETYPE", info.codec->name);
 
     if (!deadbeef->is_local_file (fname)) {
         deadbeef->plt_set_item_duration (plt, it, -1);
@@ -753,7 +691,7 @@ ffmpeg_insert (ddb_playlist_t *plt, DB_playItem_t *after, const char *fname) {
     }
 
     // add metainfo
-    ffmpeg_read_metadata_internal (it, fctx);
+    ffmpeg_read_metadata_internal (it, info.format_context);
     
     int64_t fsize = -1;
 
@@ -771,7 +709,7 @@ ffmpeg_insert (ddb_playlist_t *plt, DB_playItem_t *after, const char *fname) {
         deadbeef->pl_add_meta (it, ":FILE_SIZE", s);
         snprintf (s, sizeof (s), "%d", bps);
         deadbeef->pl_add_meta (it, ":BPS", s);
-        snprintf (s, sizeof (s), "%d", ctx->channels);
+        snprintf (s, sizeof (s), "%d", info.codec_context->channels);
         deadbeef->pl_add_meta (it, ":CHANNELS", s);
         snprintf (s, sizeof (s), "%d", samplerate);
         deadbeef->pl_add_meta (it, ":SAMPLERATE", s);
@@ -781,8 +719,7 @@ ffmpeg_insert (ddb_playlist_t *plt, DB_playItem_t *after, const char *fname) {
     }
 
     // free decoder
-    avcodec_close (ctx);
-    avformat_close_input (&fctx);
+    _free_info_data (&info);
 
     DB_playItem_t *cue = deadbeef->plt_process_cue (plt, after, it, totalsamples, samplerate);
     if (cue) {
@@ -794,6 +731,9 @@ ffmpeg_insert (ddb_playlist_t *plt, DB_playItem_t *after, const char *fname) {
     after = deadbeef->plt_insert_item (plt, after, it);
     deadbeef->pl_item_unref (it);
     return after;
+error:
+    _free_info_data (&info);
+    return NULL;
 }
 
 static int
@@ -909,9 +849,6 @@ ffmpeg_message (uint32_t id, uintptr_t ctx, uint32_t p1, uint32_t p2) {
 static int
 ffmpeg_start (void) {
     ffmpeg_init_exts ();
-#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(54, 6, 0)
-    avcodec_register_all ();
-#endif
     av_register_all ();
     return 0;
 }
@@ -927,68 +864,49 @@ ffmpeg_stop (void) {
 
 int
 ffmpeg_read_metadata (DB_playItem_t *it) {
+    int err = -1;
+
     trace ("ffmpeg_read_metadata: fname %s\n", deadbeef->pl_find_meta (it, ":URI"));
-    AVCodec *codec = NULL;
-    AVCodecContext *ctx = NULL;
-    AVFormatContext *fctx = NULL;
+    ffmpeg_info_t info = {0};
     int ret;
     char *uri = NULL;
     int i;
 
     deadbeef->pl_lock ();
-    {
-        const char *fname = deadbeef->pl_find_meta (it, ":URI");
-#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(54, 6, 0)
-        uri = strdupa (fname);
-#else
-        int l = strlen (fname);
-        uri = alloca (l + sizeof (FF_PROTOCOL_NAME) + 1);
-
-        // construct uri
-        memcpy (uri, FF_PROTOCOL_NAME, sizeof (FF_PROTOCOL_NAME)-1);
-        memcpy (uri + sizeof (FF_PROTOCOL_NAME)-1, ":", 1);
-        memcpy (uri + sizeof (FF_PROTOCOL_NAME), fname, l);
-        uri[sizeof (FF_PROTOCOL_NAME) + l] = 0;
-#endif
-    }
+    const char *fname = deadbeef->pl_find_meta (it, ":URI");
+    uri = strdupa (fname);
     deadbeef->pl_unlock ();
     trace ("ffmpeg: uri: %s\n", uri);
 
     // open file
-    if ((ret = avformat_open_input(&fctx, uri, NULL, NULL)) < 0) {
-        trace ("fctx is %p, ret %d/%s", fctx, ret, strerror(-ret));
+    if ((ret = avformat_open_input(&info.format_context, uri, NULL, NULL)) < 0) {
+        trace ("fctx is %p, ret %d/%s", info.format_context, ret, strerror(-ret));
         return -1;
     }
 
-    avformat_find_stream_info(fctx, NULL);
-    for (i = 0; i < fctx->nb_streams; i++)
-    {
-        ctx = fctx->streams[i]->codec;
-        if (ctx->codec_type == AVMEDIA_TYPE_AUDIO)
-        {
-            codec = avcodec_find_decoder(ctx->codec_id);
-            if (codec != NULL)
-                break;
+    avformat_find_stream_info(info.format_context, NULL);
+    for (i = 0; i < info.format_context->nb_streams; i++) {
+        if (_get_audio_codec_from_stream(info.format_context, i, &info)) {
+            break;
         }
     }
-    if (codec == NULL)
-    {
+    if (info.codec == NULL) {
         trace ("ffmpeg can't decode %s\n", deadbeef->pl_find_meta (it, ":URI"));
-        avformat_close_input (&fctx);
-        return -1;
+        goto error;
     }
-    if (avcodec_open2 (ctx, codec, NULL) < 0) {
+    if (avcodec_open2 (info.codec_context, info.codec, NULL) < 0) {
         trace ("ffmpeg: avcodec_open2 failed\n");
-        avformat_close_input (&fctx);
-        return -1;
+        goto error;
     }
 
     deadbeef->pl_delete_all_meta (it);
-    ffmpeg_read_metadata_internal (it, fctx);
+    ffmpeg_read_metadata_internal (it, info.format_context);
 
-    avformat_close_input (&fctx);
+    err = 0;
+error:
+    _free_info_data(&info);
 
-    return 0;
+    return err;
 }
 
 static const char settings_dlg[] =
@@ -997,15 +915,17 @@ static const char settings_dlg[] =
 ;
 
 // define plugin interface
-static DB_decoder_t plugin = {
-    DDB_PLUGIN_SET_API_VERSION
-    .plugin.version_major = 1,
-    .plugin.version_minor = 2,
-    .plugin.type = DB_PLUGIN_DECODER,
-    .plugin.id = "ffmpeg",
-    .plugin.name = "FFMPEG audio player",
-    .plugin.descr = "decodes audio formats using FFMPEG libavcodec",
-    .plugin.copyright = 
+
+static ddb_decoder2_t plugin = {
+    .decoder.plugin.api_vmajor = DB_API_VERSION_MAJOR,
+    .decoder.plugin.api_vminor = DB_API_VERSION_MINOR,
+    .decoder.plugin.version_major = 1,
+    .decoder.plugin.version_minor = 2,
+    .decoder.plugin.type = DB_PLUGIN_DECODER,
+    .decoder.plugin.id = "ffmpeg",
+    .decoder.plugin.name = "FFMPEG audio player",
+    .decoder.plugin.descr = "decodes audio formats using FFMPEG libavcodec",
+    .decoder.plugin.copyright =
         "Copyright (C) 2009-2013 Alexey Yakovenko <waker@users.sourceforge.net>\n"
         "\n"
         "This program is free software; you can redistribute it and/or\n"
@@ -1022,20 +942,21 @@ static DB_decoder_t plugin = {
         "along with this program; if not, write to the Free Software\n"
         "Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.\n"
     ,
-    .plugin.website = "http://deadbeef.sf.net",
-    .plugin.start = ffmpeg_start,
-    .plugin.stop = ffmpeg_stop,
-    .plugin.configdialog = settings_dlg,
-    .plugin.message = ffmpeg_message,
-    .open = ffmpeg_open,
-    .init = ffmpeg_init,
-    .free = ffmpeg_free,
-    .read = ffmpeg_read,
-    .seek = ffmpeg_seek,
-    .seek_sample = ffmpeg_seek_sample,
-    .insert = ffmpeg_insert,
-    .read_metadata = ffmpeg_read_metadata,
-    .exts = (const char **)exts,
+    .decoder.plugin.website = "http://deadbeef.sf.net",
+    .decoder.plugin.start = ffmpeg_start,
+    .decoder.plugin.stop = ffmpeg_stop,
+    .decoder.plugin.configdialog = settings_dlg,
+    .decoder.plugin.message = ffmpeg_message,
+    .decoder.open = ffmpeg_open,
+    .decoder.init = ffmpeg_init,
+    .decoder.free = ffmpeg_free,
+    .decoder.read = ffmpeg_read,
+    .decoder.seek = ffmpeg_seek,
+    .decoder.seek_sample = ffmpeg_seek_sample,
+    .decoder.insert = ffmpeg_insert,
+    .decoder.read_metadata = ffmpeg_read_metadata,
+    .decoder.exts = (const char **)exts,
+    .seek_sample64 = ffmpeg_seek_sample64,
 };
 
 DB_plugin_t *
