@@ -41,8 +41,11 @@ struct _DdbCellRendererTextMultilinePrivate {
     gulong focus_out_id;
 
     gulong populate_popup_id;
-    gulong entry_menu_popdown_timeout;
+    guint entry_menu_popdown_timeout;
     gboolean in_entry_menu;
+
+    guint is_mult_column;
+    guint value_column;
 };
 
 struct _DdbCellEditableTextViewPrivate {
@@ -63,6 +66,8 @@ DdbCellEditableTextView* ddb_cell_editable_text_view_new (void);
 DdbCellEditableTextView* ddb_cell_editable_text_view_construct (GType object_type);
 static void ddb_cell_editable_text_view_finalize (GObject* obj);
 GType ddb_cell_renderer_text_multiline_get_type (void);
+DdbCellRendererTextMultiline* ddb_cell_renderer_text_multiline_construct (GType object_type);
+
 #define DDB_CELL_RENDERER_TEXT_MULTILINE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), DDB_TYPE_CELL_RENDERER_TEXT_MULTILINE, DdbCellRendererTextMultilinePrivate))
 #define DDB_CELL_EDITABLE_TEXT_VIEW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), DDB_TYPE_CELL_EDITABLE_TEXT_VIEW, DdbCellEditableTextViewPrivate))
 enum  {
@@ -70,7 +75,15 @@ enum  {
 };
 static void ddb_cell_renderer_text_multiline_gtk_cell_renderer_text_editing_done (DdbCellEditableTextView* entry, DdbCellRendererTextMultiline* _self_);
 static gboolean ddb_cell_renderer_text_multiline_gtk_cell_renderer_focus_out_event (DdbCellEditableTextView* entry, GdkEvent* event, DdbCellRendererTextMultiline* _self_);
-static GtkCellEditable* ddb_cell_renderer_text_multiline_real_start_editing (GtkCellRenderer* base, GdkEvent* event, GtkWidget* widget, const gchar* path, GdkRectangle* background_area, GdkRectangle* cell_area, GtkCellRendererState flags);
+static GtkCellEditable* ddb_cell_renderer_text_multiline_real_start_editing (
+                                                                             GtkCellRenderer      *cell,
+                                                                             GdkEvent             *event,
+                                                                             GtkWidget            *widget,
+                                                                             const gchar          *path,
+                                                                             const GdkRectangle   *background_area,
+                                                                             const GdkRectangle   *cell_area,
+                                                                             GtkCellRendererState  flags
+                                                                             );
 DdbCellRendererTextMultiline* ddb_cell_renderer_text_multiline_new (void);
 DdbCellRendererTextMultiline* ddb_cell_renderer_text_multiline_construct (GType object_type);
 static void ddb_cell_renderer_text_multiline_finalize (GObject* obj);
@@ -114,12 +127,6 @@ static void ddb_cell_editable_text_view_real_start_editing (GtkCellEditable* bas
     self = (DdbCellEditableTextView*) base;
 }
 
-
-#if GTK_CHECK_VERSION(2,20,0)
-static void ddb_cell_editable_text_view_real_editing_canceled (GtkCellRenderer* base) {
-}
-#endif
-
 DdbCellEditableTextView* ddb_cell_editable_text_view_construct (GType object_type) {
     DdbCellEditableTextView * self;
     self = g_object_newv (object_type, 0, NULL);
@@ -231,7 +238,7 @@ static void ddb_cell_editable_text_view_finalize (GObject* obj) {
 
 GType ddb_cell_editable_text_view_get_type (void) {
     static volatile gsize ddb_cell_editable_text_view_type_id__volatile = 0;
-    if (g_once_init_enter (&ddb_cell_editable_text_view_type_id__volatile)) {
+    if (g_once_init_enter ((gsize *)(&ddb_cell_editable_text_view_type_id__volatile))) {
         static const GTypeInfo g_define_type_info = { sizeof (DdbCellEditableTextViewClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) ddb_cell_editable_text_view_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DdbCellEditableTextView), 0, (GInstanceInitFunc) ddb_cell_editable_text_view_instance_init, NULL };
         static const GInterfaceInfo gtk_cell_editable_info = { (GInterfaceInitFunc) ddb_cell_editable_text_view_gtk_cell_editable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
         GType ddb_cell_editable_text_view_type_id;
@@ -370,7 +377,15 @@ ddb_cell_renderer_text_multiline_populate_popup (GtkEntry *entry,
             G_CALLBACK (ddb_cell_renderer_text_multiline_popup_unmap), data);
 }
 
-static GtkCellEditable* ddb_cell_renderer_text_multiline_real_start_editing (GtkCellRenderer* base, GdkEvent* event, GtkWidget* widget, const gchar* path, GdkRectangle* background_area, GdkRectangle* cell_area, GtkCellRendererState flags) {
+static GtkCellEditable* ddb_cell_renderer_text_multiline_real_start_editing (
+                                                                             GtkCellRenderer      *cell,
+                                                                             GdkEvent             *event,
+                                                                             GtkWidget            *widget,
+                                                                             const gchar          *path,
+                                                                             const GdkRectangle   *background_area,
+                                                                             const GdkRectangle   *cell_area,
+                                                                             GtkCellRendererState  flags
+                                                                             ) {
     DdbCellRendererTextMultiline * self;
     DdbCellEditableTextView* textview;
     GValue v = {0};
@@ -385,7 +400,7 @@ static GtkCellEditable* ddb_cell_renderer_text_multiline_real_start_editing (Gtk
     gint mult;
 
 
-    self = (DdbCellRendererTextMultiline*) base;
+    self = (DdbCellRendererTextMultiline*) cell;
     g_return_val_if_fail (widget != NULL, NULL);
     g_return_val_if_fail (path != NULL, NULL);
     g_return_val_if_fail (background_area != NULL, NULL);
@@ -401,8 +416,8 @@ static GtkCellEditable* ddb_cell_renderer_text_multiline_real_start_editing (Gtk
     store = GTK_LIST_STORE (gtk_tree_view_get_model (tv));
     _g_object_ref0 (store);
     gtk_tree_model_get_iter ((GtkTreeModel*) store, &iter, p);
-    G_IS_VALUE (&v) ? (g_value_unset (&v), NULL) : NULL;
-    gtk_tree_model_get_value ((GtkTreeModel*) store, &iter, 3, &v);
+    G_IS_VALUE (&v) ? ((void)(g_value_unset (&v)), NULL) : NULL;
+    gtk_tree_model_get_value ((GtkTreeModel*) store, &iter, self->priv->is_mult_column, &v);
     mult = g_value_get_int (&v);
     _g_object_unref0 (self->priv->entry);
     self->priv->entry = textview = ddb_cell_editable_text_view_new ();
@@ -411,7 +426,7 @@ static GtkCellEditable* ddb_cell_renderer_text_multiline_real_start_editing (Gtk
     buf = gtk_text_buffer_new (NULL);
     if (!mult) {
         GValue full_textv = {0};
-        gtk_tree_model_get_value ((GtkTreeModel*) store, &iter, 4, &full_textv);
+        gtk_tree_model_get_value ((GtkTreeModel*) store, &iter, self->priv->value_column, &full_textv);
         if (G_IS_VALUE (&full_textv)) {
             const char *full_text = g_value_get_string (&full_textv);
             renderer_text = strdup (full_text);
@@ -447,9 +462,10 @@ static GtkCellEditable* ddb_cell_renderer_text_multiline_real_start_editing (Gtk
 
     _g_object_unref0 (buf);
     gtk_widget_set_size_request ((GtkWidget*) textview, cell_area->width, cell_area->height);
+    gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (textview), GTK_WRAP_WORD);
     gtk_widget_show ((GtkWidget*) textview);
     result = GTK_CELL_EDITABLE (textview);
-    G_IS_VALUE (&v) ? (g_value_unset (&v), NULL) : NULL;
+    G_IS_VALUE (&v) ? ((void)(g_value_unset (&v)), NULL) : NULL;
     _g_object_unref0 (store);
     _g_object_unref0 (tv);
     _gtk_tree_path_free0 (p);
@@ -475,15 +491,27 @@ DdbCellRendererTextMultiline* ddb_cell_renderer_text_multiline_new (void) {
 static void ddb_cell_renderer_text_multiline_class_init (DdbCellRendererTextMultilineClass * klass) {
     ddb_cell_renderer_text_multiline_parent_class = g_type_class_peek_parent (klass);
     g_type_class_add_private (klass, sizeof (DdbCellRendererTextMultilinePrivate));
+#if GTK_CHECK_VERSION(3,0,0)
     GTK_CELL_RENDERER_CLASS (klass)->start_editing = ddb_cell_renderer_text_multiline_real_start_editing;
+#else
+    // GTK2 uses a slightly different, but compatible declaration without const modifiers
+    GTK_CELL_RENDERER_CLASS (klass)->start_editing = (GtkCellEditable *(*) (GtkCellRenderer *, GdkEvent *, GtkWidget *, const gchar *, GdkRectangle *, GdkRectangle *cell_area, GtkCellRendererState))ddb_cell_renderer_text_multiline_real_start_editing;
+#endif
     G_OBJECT_CLASS (klass)->finalize = ddb_cell_renderer_text_multiline_finalize;
 }
 
 
 static void ddb_cell_renderer_text_multiline_instance_init (DdbCellRendererTextMultiline * self) {
     self->priv = DDB_CELL_RENDERER_TEXT_MULTILINE_GET_PRIVATE (self);
+    // Default column indexes for Track Properties
+    self->priv->is_mult_column = 3;
+    self->priv->value_column = 4;
 }
 
+void ddb_cell_renderer_text_multiline_set_columns(DdbCellRendererTextMultiline *self, guint is_mult, guint value) {
+    self->priv->is_mult_column = is_mult;
+    self->priv->value_column = value;
+}
 
 static void ddb_cell_renderer_text_multiline_finalize (GObject* obj) {
     DdbCellRendererTextMultiline * self;
@@ -495,7 +523,7 @@ static void ddb_cell_renderer_text_multiline_finalize (GObject* obj) {
 
 GType ddb_cell_renderer_text_multiline_get_type (void) {
     static volatile gsize ddb_cell_renderer_text_multiline_type_id__volatile = 0;
-    if (g_once_init_enter (&ddb_cell_renderer_text_multiline_type_id__volatile)) {
+    if (g_once_init_enter ((gsize *)(&ddb_cell_renderer_text_multiline_type_id__volatile))) {
         static const GTypeInfo g_define_type_info = { sizeof (DdbCellRendererTextMultilineClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) ddb_cell_renderer_text_multiline_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DdbCellRendererTextMultiline), 0, (GInstanceInitFunc) ddb_cell_renderer_text_multiline_instance_init, NULL };
         GType ddb_cell_renderer_text_multiline_type_id;
         ddb_cell_renderer_text_multiline_type_id = g_type_register_static (GTK_TYPE_CELL_RENDERER_TEXT, "DdbCellRendererTextMultiline", &g_define_type_info, 0);
