@@ -22,19 +22,19 @@
 */
 
 int
-action_scan_all_tracks_without_rg_handler (struct DB_plugin_action_s *action, int ctx);
+action_scan_all_tracks_without_rg_handler (struct DB_plugin_action_s *action, ddb_action_context_t ctx);
 
 int
-action_rg_scan_per_file_handler (struct DB_plugin_action_s *action, int ctx);
+action_rg_scan_per_file_handler (struct DB_plugin_action_s *action, ddb_action_context_t ctx);
 
 int
-action_rg_remove_info_handler (struct DB_plugin_action_s *action, int ctx);
+action_rg_remove_info_handler (struct DB_plugin_action_s *action, ddb_action_context_t ctx);
 
 int
-action_rg_scan_selection_as_albums_handler (struct DB_plugin_action_s *action, int ctx);
+action_rg_scan_selection_as_albums_handler (struct DB_plugin_action_s *action, ddb_action_context_t ctx);
 
 int
-action_rg_scan_selection_as_album_handler (struct DB_plugin_action_s *action, int ctx);
+action_rg_scan_selection_as_album_handler (struct DB_plugin_action_s *action, ddb_action_context_t ctx);
 
 
 
