@@ -24,10 +24,7 @@
 #ifndef __SEARCH_H
 #define __SEARCH_H
 
-#include "ddblistview.h"
-
-extern struct playItem_s *search_current;
-extern int search_count;
+#include "playlist/ddblistview.h"
 
 void
 search_start (void);
@@ -35,17 +32,13 @@ search_start (void);
 void
 search_destroy (void);
 
-// should be called whenever playlist was changed
-void
-search_refresh (void);
-
-void
-search_redraw (void);
-
 int
-search_get_idx (DdbListviewIter it);
+search_message (uint32_t id, uintptr_t ctx, uint32_t p1, uint32_t p2);
 
 void
 search_playlist_init (GtkWidget *widget);
 
+void
+search_submit_refresh (void);
+
 #endif // __SEARCH_H
