@@ -16,21 +16,27 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
+#include <Block.h>
 #include <dbus/dbus.h>
+#include <dispatch/dispatch.h>
 #include "../../deadbeef.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include "../../gettext.h"
-#include "../artwork-legacy/artwork.h"
+#include "../artwork/artwork.h"
 
 #define E_NOTIFICATION_BUS_NAME "org.freedesktop.Notifications"
 #define E_NOTIFICATION_INTERFACE "org.freedesktop.Notifications"
 #define E_NOTIFICATION_PATH "/org/freedesktop/Notifications"
 
-DB_functions_t *deadbeef;
-DB_misc_t plugin;
-DB_artwork_plugin_t *artwork_plugin;
+static DB_functions_t *deadbeef;
+static DB_misc_t plugin;
+static ddb_artwork_plugin_t *artwork_plugin;
+static dispatch_queue_t queue;
+static DB_playItem_t *last_track = NULL;
+static time_t request_timer = 0;
+static int terminate;
 
 static dbus_uint32_t replaces_id = 0;
 
@@ -41,9 +47,10 @@ static char *tf_title;
 static char *tf_content;
 
 static void
-notify_thread (void *ctx) {
+show_notification (DB_playItem_t *track, char *image_filename);
 
-    DBusMessage *msg = (DBusMessage*) ctx;
+static void
+notify_send (DBusMessage *msg) {
     DBusMessage *reply = NULL;
 
     DBusError error;
@@ -53,7 +60,7 @@ notify_thread (void *ctx) {
         fprintf(stderr, "connection failed: %s",error.message);
         dbus_error_free(&error);
         dbus_message_unref (msg);
-        deadbeef->thread_exit(NULL);
+        return;
     }
 
     reply = dbus_connection_send_with_reply_and_block (conn, msg, -1, &error);
@@ -61,7 +68,7 @@ notify_thread (void *ctx) {
         fprintf(stderr, "send_with_reply_and_block error: (%s)\n", error.message); 
         dbus_error_free(&error);
         dbus_message_unref (msg);
-        deadbeef->thread_exit(NULL);
+        return;
     }
 
     if (reply != NULL) {
@@ -86,8 +93,6 @@ notify_thread (void *ctx) {
 
     dbus_message_unref (msg);
     dbus_connection_unref (conn);
-    deadbeef->thread_exit(NULL);
-
 }
 
 static void
@@ -150,28 +155,15 @@ esc_xml (const char *cmd, char *esc, int size) {
     *dst = 0;
 }
 
-static void show_notification (DB_playItem_t *track);
-static DB_playItem_t *last_track = NULL;
-static time_t request_timer = 0;
-
 static void
-cover_avail_callback (const char *fname, const char *artist, const char *album, void *user_data) {
-    if (!fname) {
-        // Give up
-        return;
-    }
-    deadbeef->pl_lock ();
-    if (last_track && (time (NULL) - request_timer < 4)) {
-        show_notification (last_track);
-    }
-    if (last_track) {
-        deadbeef->pl_item_unref (last_track);
-        last_track = NULL;
-    }
-    deadbeef->pl_unlock ();
+_cover_loaded_callback (int error, ddb_cover_query_t *query, ddb_cover_info_t *cover) {
+    void (^completion_block)(int error, ddb_cover_query_t *query, ddb_cover_info_t *cover) = (void (^)(int error, ddb_cover_query_t *query, ddb_cover_info_t *cover))query->user_data;
+    completion_block(error, query, cover);
+    Block_release(completion_block);
 }
 
-static void show_notification (DB_playItem_t *track) {
+static void
+show_notification (DB_playItem_t *track, char *image_filename) {
     char title[1024];
     char content[1024];
 
@@ -200,20 +192,39 @@ static void show_notification (DB_playItem_t *track) {
 
     const char *v_appname = "DeaDBeeF";
     dbus_uint32_t v_id = 0;
-    char *v_iconname = NULL;
-    if (deadbeef->conf_get_int("notify.albumart", 0) && artwork_plugin) {
-        deadbeef->pl_lock ();
-        const char *album = deadbeef->pl_find_meta (track, "album");
-        const char *artist = deadbeef->pl_find_meta (track, "artist");
-        const char *fname = deadbeef->pl_find_meta (track, ":URI");
-        if (!album || !*album) {
-            album = deadbeef->pl_find_meta (track, "title");
-        }
-        v_iconname = artwork_plugin->get_album_art (fname, artist, album, deadbeef->conf_get_int ("notify.albumart_size", 64), cover_avail_callback, NULL);
-        deadbeef->pl_unlock ();
+    char *v_iconname = image_filename;
+    if (!v_iconname && deadbeef->conf_get_int("notify.albumart", 0) && artwork_plugin) {
+
+        ddb_cover_query_t *query = calloc (sizeof (ddb_cover_query_t), 1);
+        query->_size = sizeof (ddb_cover_query_t);
+        query->track = track;
+        deadbeef->pl_item_ref (track);
+        query->source_id = 0;
+
+        void (^completion_block)(int error, ddb_cover_query_t *query, ddb_cover_info_t *cover) = ^(int error, ddb_cover_query_t *query, ddb_cover_info_t *cover) {
+            if (!(query->flags & DDB_ARTWORK_FLAG_CANCELLED) && cover != NULL && cover->image_filename) {
+                // redisplay notification with the album art
+                char *image_filename = strdup (cover->image_filename);
+                ddb_playItem_t *track = query->track;
+                deadbeef->pl_item_ref (track);
+                dispatch_async (queue, ^{
+                    show_notification (track, image_filename);
+                    free (image_filename);
+                    deadbeef->pl_item_unref (track);
+                });
+            }
+            deadbeef->pl_item_unref (query->track);
+            free (query);
+            if (cover != NULL) {
+                artwork_plugin->cover_info_release (cover);
+            }
+        };
+
+        query->user_data = (dispatch_block_t)Block_copy(completion_block);
+        artwork_plugin->cover_get (query, _cover_loaded_callback);
     }
     if (!v_iconname) {
-        v_iconname = strdup ("deadbeef");
+        v_iconname = "deadbeef";
     }
     const char *v_summary = title;
     const char *v_body = esc_content;
@@ -239,15 +250,7 @@ static void show_notification (DB_playItem_t *track) {
 
     dbus_message_iter_append_basic(&iter, DBUS_TYPE_INT32, &v_timeout);
 
-    intptr_t tid = 0;
-    if ((tid=deadbeef->thread_start(notify_thread, msg)) != 0) {
-        dbus_message_ref (msg);
-        deadbeef->thread_detach (tid);  
-    }
-    dbus_message_unref (msg);
-    if (v_iconname) {
-        free (v_iconname);
-    }
+    notify_send (msg);
 }
 
 static int
@@ -255,7 +258,15 @@ on_songstarted (ddb_event_track_t *ev) {
     if (ev->track && deadbeef->conf_get_int ("notify.enable", 0)) {
         DB_playItem_t *track = ev->track;
         if (track) {
-            show_notification (track);
+            deadbeef->pl_item_ref (track);
+            if (terminate) {
+                deadbeef->pl_item_unref (track);
+                return 0;
+            }
+            dispatch_async (queue, ^{
+                show_notification (track, NULL);
+                deadbeef->pl_item_unref (track);
+            });
         }
     }
     return 0;
@@ -304,6 +315,7 @@ import_legacy_tf (const char *key_from, const char *key_to) {
 
 int
 notify_start (void) {
+    queue = dispatch_queue_create("OSDNotifyQueue", NULL);
     import_legacy_tf ("notify.format", "notify.format_title_tf");
     import_legacy_tf ("notify.format_content", "notify.format_content_tf");
     return 0;
@@ -311,6 +323,10 @@ notify_start (void) {
 
 int
 notify_stop (void) {
+    terminate = 1;
+    dispatch_release (queue);
+    queue = NULL;
+
     deadbeef->pl_lock ();
     if (last_track) {
         deadbeef->pl_item_unref (last_track);
@@ -331,7 +347,7 @@ notify_stop (void) {
 
 static int
 notify_connect (void) {
-    artwork_plugin = (DB_artwork_plugin_t *)deadbeef->plug_get_for_id ("artwork");
+    artwork_plugin = (ddb_artwork_plugin_t *)deadbeef->plug_get_for_id ("artwork2");
     return 0;
 }
 
@@ -349,7 +365,7 @@ static const char settings_dlg[] =
     "property \"Album art size (px)\" entry notify.albumart_size 64;\n"
 ;
 
-DB_misc_t plugin = {
+static DB_misc_t plugin = {
     DDB_PLUGIN_SET_API_VERSION
     .plugin.type = DB_PLUGIN_MISC,
     .plugin.version_major = 1,
