@@ -53,13 +53,6 @@ enum
     COL_DURATION,
 };
 
-// copied from gktui.c
-static void
-playlist_set_curr (int playlist) {
-    deadbeef->plt_set_curr_idx (playlist);
-    deadbeef->conf_set_int ("playlist.current", playlist);
-}
-
 // copied from gtkui.c
 static int
 add_new_playlist (void) {
@@ -120,7 +113,7 @@ on_pltbrowser_row_inserted (GtkTreeModel *tree_model, GtkTreePath *path, GtkTree
     int *indices = gtk_tree_path_get_indices (path);
     int idx = *indices;
     if (idx > plt->last_selected) {
-        idx--;
+        idx--; // this is necessary, because in this case the rows will shift "up".
     }
     if (idx == plt->last_selected) {
         return;
@@ -129,7 +122,7 @@ on_pltbrowser_row_inserted (GtkTreeModel *tree_model, GtkTreePath *path, GtkTree
     deadbeef->plt_move (plt->last_selected, idx);
     plt->last_selected = idx;
     deadbeef->plt_set_curr_idx (idx);
-    deadbeef->sendmessage (DB_EV_PLAYLISTSWITCHED, 0, 0, 0);
+    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_POSITION, 0);
 }
 
 static gboolean
@@ -216,6 +209,7 @@ fill_pltbrowser_rows (gpointer user_data)
             snprintf (totaltime_str, sizeof (totaltime_str), "%d:%02d:%02d", hourtotal, mintotal, sectotal);
         }
         else {
+            // NOTE: localizable because of the "d" suffix for days
             snprintf (totaltime_str, sizeof (totaltime_str), _("%dd %d:%02d:%02d"), daystotal, hourtotal, mintotal, sectotal);
         }
 
@@ -227,7 +221,6 @@ fill_pltbrowser_rows (gpointer user_data)
 
 static gboolean
 update_pltbrowser_cb (gpointer data) {
-    w_pltbrowser_t *w = data;
     fill_pltbrowser_rows (data);
     return FALSE;
 }
@@ -529,7 +522,6 @@ static gboolean
 on_pltbrowser_column_clicked (GtkTreeViewColumn     *col,
                               gpointer         user_data)
 {
-    w_pltbrowser_t *w = user_data;
     GtkWidget *view = gtk_tree_view_column_get_tree_view (GTK_TREE_VIEW_COLUMN (col));
     int order = gtk_tree_view_column_get_sort_order (GTK_TREE_VIEW_COLUMN (col));
 
@@ -586,23 +578,6 @@ on_pltbrowser_header_clicked (GtkWidget       *widget,
     return FALSE;
 }
 
-/*
-static void
-plt_get_title_wrapper (int plt, char *buffer, int len) {
-    if (plt == -1) {
-        strcpy (buffer, "");
-        return;
-    }
-    ddb_playlist_t *p = deadbeef->plt_get_for_idx (plt);
-    deadbeef->plt_get_title (p, buffer, len);
-    deadbeef->plt_unref (p);
-    char *end;
-    if (!g_utf8_validate (buffer, -1, (const gchar **)&end)) {
-        *end = 0;
-    }
-}
-*/
-
 static gboolean
 on_pltbrowser_key_press_event (GtkWidget *widget,
                                GdkEventKey  *event,
@@ -634,61 +609,9 @@ on_pltbrowser_key_press_event (GtkWidget *widget,
             }
         }
     }
-
-    //if (event->keyval == GDK_F2) {
-    //    // rename selected playlist
-    //    GtkTreePath *path;
-    //    GtkTreeViewColumn *col;
-    //    gtk_tree_view_get_cursor (GTK_TREE_VIEW (w->tree), &path, NULL);
-    //    col = gtk_tree_view_get_column (GTK_TREE_VIEW (w->tree), COL_NAME);
-    //    if (!path || !col) {
-    //        return FALSE;
-    //    }
-    //    // start editing
-    //    gtk_tree_view_set_cursor_on_cell (GTK_TREE_VIEW (w->tree), path, col, NULL, TRUE);
-    //}
     return FALSE;
 }
 
-/*
-static void
-on_pltbrowser_cell_edititing_started (GtkCellRenderer *renderer,
-                                      GtkCellEditable *editable,
-                                      gchar           *path,
-                                      gpointer         user_data)
-{
-    w_pltbrowser_t *w = user_data;
-    if (deadbeef->conf_get_int ("gtkui.pltbrowser.highlight_curr_plt", 0)
-            && GTK_IS_ENTRY (editable)) {
-        // we need to get rid of the "(playing)" string before editing the playlist name
-        int row = get_treeview_cursor_pos (GTK_TREE_VIEW (w->tree));
-        if (row >= 0) {
-            GtkEntry *entry = GTK_ENTRY (editable);
-            char t[1000];
-            plt_get_title_wrapper (row, t, sizeof (t));
-            gtk_entry_set_text (GTK_ENTRY (entry), t);
-        }
-    }
-}
-
-static void
-on_pltbrowser_cell_edited (GtkCellRendererText *cell,
-                           gchar               *path_string,
-                           gchar               *new_text,
-                           gpointer             user_data)
-{
-    w_pltbrowser_t *w = user_data;
-    int row = get_treeview_cursor_pos (GTK_TREE_VIEW (w->tree));
-    if (row >= 0) {
-        deadbeef->pl_lock ();
-        ddb_playlist_t *p = deadbeef->plt_get_for_idx (row);
-        deadbeef->plt_set_title (p, new_text);
-        deadbeef->plt_unref (p);
-        deadbeef->pl_unlock ();
-    }
-}
-*/
-
 static GtkTreeViewColumn *
 add_treeview_column (w_pltbrowser_t *w, GtkTreeView *tree, int pos, int expand, int align_right, const char *title, int is_pixbuf)
 {
@@ -821,15 +744,13 @@ on_pltbrowser_button_press_end_event         (GtkWidget       *widget,
                 // new tab
                 int playlist = add_new_playlist ();
                 if (playlist != -1) {
-                    playlist_set_curr (playlist);
+                    deadbeef->plt_set_curr_idx (playlist);
                 }
                 return TRUE;
             }
             else if (deadbeef->conf_get_int ("gtkui.pltbrowser.mmb_delete_playlist", 0)) {
                 if (row_clicked != -1) {
                     deadbeef->plt_remove (row_clicked);
-                    int playlist = deadbeef->plt_get_curr_idx ();
-                    deadbeef->conf_set_int ("playlist.current", playlist);
                 }
             }
         }
@@ -841,7 +762,7 @@ on_pltbrowser_button_press_end_event         (GtkWidget       *widget,
                 // new tab
                 int playlist = add_new_playlist ();
                 if (playlist != -1) {
-                    playlist_set_curr (playlist);
+                    deadbeef->plt_set_curr_idx (playlist);
                 }
                 return TRUE;
             }
@@ -924,7 +845,7 @@ w_pltbrowser_create (void) {
     w->ri_id = g_signal_connect ((gpointer) store, "row_inserted", G_CALLBACK (on_pltbrowser_row_inserted), w);
 
     gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (w->tree), TRUE);
-    GtkTreeViewColumn *col1 = add_treeview_column (w, GTK_TREE_VIEW (w->tree), COL_NAME, 1, 0, _("Name"), 0);
+    add_treeview_column (w, GTK_TREE_VIEW (w->tree), COL_NAME, 1, 0, _("Name"), 0);
 
     int show_playing_column = deadbeef->conf_get_int ("gtkui.pltbrowser.show_playing_column", 0);
     w->col_playing = add_treeview_column (w, GTK_TREE_VIEW (w->tree), COL_PLAYING, 0, 1, _("♫"), 1);
@@ -1014,11 +935,7 @@ static DB_misc_t plugin = {
 #else
     .plugin.id = "pltbrowser_gtk2",
 #endif
-#if GTK_CHECK_VERSION(3,0,0)
-    .plugin.name = "Playlist browser GTK3",
-#else
-    .plugin.name = "Playlist browser GTK2",
-#endif
+    .plugin.name = "Playlist Browser",
     .plugin.descr = "Use View -> Design Mode to add playlist browser into main window",
     .plugin.copyright = 
         "Playlist browser widget plugin for DeaDBeeF Player\n"
