@@ -47,14 +47,16 @@
 #include "volume.h"
 #include "vfs.h"
 #include "premix.h"
-#include "fft.h"
 #include "handler.h"
 #include "plugins/libparser/parser.h"
 #include "strdupa.h"
 #include "playqueue.h"
 #include "streamreader.h"
+#include "decodedblock.h"
 #include "dsp.h"
 #include "playmodes.h"
+#include "viz.h"
+#include "fft.h"
 
 #ifdef trace
 #undef trace
@@ -83,6 +85,8 @@ static int conf_streamer_use_dependent_samplerate = 0;
 static int conf_streamer_samplerate = 44100;
 static int conf_streamer_samplerate_mult_48 = 48000;
 static int conf_streamer_samplerate_mult_44 = 44100;
+static float conf_format_silence = -1.f;
+static float conf_playback_buffer_size = 0.3f;
 
 static int trace_bufferfill = 0;
 
@@ -92,7 +96,6 @@ static int stop_after_album = 0;
 static int streaming_terminate;
 
 static uintptr_t mutex;
-static uintptr_t wdl_mutex; // wavedata listener
 
 static float last_seekpos = -1;
 
@@ -109,7 +112,6 @@ static time_t started_timestamp; // result of calling time(NULL)
 static playItem_t *streaming_track;
 static playItem_t *last_played; // this is the last track that was played, should avoid setting this to NULL
 
-static int _format_change_wait;
 static ddb_waveformat_t prev_output_format; // last format that was sent to output via streamer_set_output_format
 static ddb_waveformat_t last_block_fmt; // input file format corresponding to the current output
 
@@ -142,32 +144,13 @@ static char *_output_buffer;
 static size_t _output_buffer_size;
 static int _outbuffer_remaining;
 
-// A buffer used in streamer_read.
-static float *_temp_audio_buffer;
-static size_t _temp_audio_buffer_size;
-
 #if defined(HAVE_XGUI) || defined(ANDROID)
 #include "equalizer.h"
 #endif
 
-static float freq_data[DDB_FREQ_BANDS * DDB_FREQ_MAX_CHANNELS];
-static float audio_data[DDB_FREQ_BANDS * 2 * DDB_FREQ_MAX_CHANNELS];
-static int audio_data_fill = 0;
-static int audio_data_channels = 0;
-
 // message queue
 static struct handler_s *handler;
 
-// visualization stuff
-typedef struct wavedata_listener_s {
-    void *ctx;
-    void (*callback)(void *ctx, ddb_audio_data_t *data);
-    struct wavedata_listener_s *next;
-} wavedata_listener_t;
-
-static wavedata_listener_t *waveform_listeners;
-static wavedata_listener_t *spectrum_listeners;
-
 #if DETECT_PL_LOCK_RC
 volatile pthread_t streamer_lock_tid = 0;
 #endif
@@ -260,9 +243,6 @@ send_songfinished (playItem_t *trk) {
 
 static void
 send_trackchanged (playItem_t *from, playItem_t *to) {
-    if (from == to) {
-        return;
-    }
     ddb_event_trackchange_t *event = (ddb_event_trackchange_t *)messagepump_event_alloc (DB_EV_SONGCHANGED);
     event->playtime = playtime;
     event->started_timestamp = started_timestamp;
@@ -299,8 +279,6 @@ streamer_start_playback (playItem_t *from, playItem_t *it) {
 
     streamer_set_playing_track (it);
     if (playing_track) {
-        playing_track->played = 1;
-
         streamer_set_last_played (playing_track);
 
         playItem_t *qnext = playqueue_getnext();
@@ -814,7 +792,7 @@ streamer_play_failed (playItem_t *failed_track) {
         streamer_set_last_played (failed_track);
         // the track has failed to be played,
         // but we want to send it down to streamreader for proper track switching etc.
-        fileinfo_curr = calloc(sizeof (DB_fileinfo_t), 1);
+        fileinfo_curr = calloc(1, sizeof (DB_fileinfo_t));
 
         fileinfo_file_vfs = NULL;
         fileinfo_file_identifier = 0;
@@ -870,6 +848,7 @@ stream_track (playItem_t *it, int startpaused) {
         pl_item_ref (from);
     }
     if (to) {
+        to->played = 1;
         pl_item_ref (to);
     }
 
@@ -1095,8 +1074,6 @@ m3u_error:
                 close (fd);
             }
 
-            it->played = 1;
-
             if (!startpaused) {
                 streamer_play_failed (it);
             }
@@ -1175,7 +1152,6 @@ m3u_error:
 
         if (!dec) {
             trace ("no decoder in playitem!\n");
-            it->played = 1;
 
             if (!startpaused) {
                 streamer_play_failed (it);
@@ -1282,7 +1258,9 @@ streamer_set_nextsong (int song, int startpaused) {
 
 void
 streamer_set_seek (float pos) {
+    streamer_lock();
     last_seekpos = pos;
+    streamer_unlock();
     handler_push (handler, STR_EV_SEEK, 0, *((uint32_t *)&pos), 0);
 }
 
@@ -1296,17 +1274,18 @@ update_stop_after_current (void) {
 }
 
 static void
-streamer_next (ddb_shuffle_t shuffle, ddb_repeat_t repeat) {
-    playItem_t *next = NULL;
+streamer_next (ddb_shuffle_t shuffle, ddb_repeat_t repeat, playItem_t *next) {
     if (playing_track) {
         if (repeat == DDB_REPEAT_SINGLE) { // song finished, loop mode is "loop 1 track"
             next = playing_track;
-            pl_item_ref (next);
         }
     }
     if (!next) {
         next = get_next_track (streaming_track, shuffle, repeat);
     }
+    else {
+        pl_item_ref (next);
+    }
     stream_track (next, 0);
     if (next) {
         pl_item_unref (next);
@@ -1406,7 +1385,9 @@ handle_track_change (playItem_t *from, playItem_t *track) {
     // otherwise the track is the first one, and playpos is pre-set
     playtime = 0;
     avg_bitrate = -1;
+    streamer_lock();
     last_seekpos = -1;
+    streamer_unlock();
     if (playing_track) {
         send_songstarted (playing_track);
     }
@@ -1486,7 +1467,7 @@ _streamer_requeue_after_current (ddb_repeat_t repeat, ddb_shuffle_t shuffle) {
         pl_item_ref (streaming_track);
     }
     streamer_unlock ();
-    streamer_next (shuffle, repeat);
+    streamer_next (shuffle, repeat, NULL);
 }
 
 static void
@@ -1522,6 +1503,10 @@ streamer_thread (void *unused) {
     uint32_t id;
     uintptr_t ctx;
     uint32_t p1, p2;
+
+    ddb_waveformat_t prev_block_fmt = {0};
+    double _add_format_silence = .0;
+
     while (!streaming_terminate) {
         struct timeval tm1;
         DB_output_t *output = plug_get_output ();
@@ -1583,26 +1568,13 @@ streamer_thread (void *unused) {
             _streamer_requeue_after_current(repeat, shuffle);
         }
 
-        if (output && output->state () == DDB_PLAYBACK_STATE_STOPPED) {
+        if (output->state () == DDB_PLAYBACK_STATE_STOPPED) {
             if (!handler_hasmessages (handler)) {
                 usleep (50000);
             }
             continue;
         }
 
-        if (_format_change_wait) {
-            streamer_lock ();
-            streamblock_t *block = streamreader_get_curr_block();
-            if (block) {
-                if (memcmp (&block->fmt, &last_block_fmt, sizeof (ddb_waveformat_t))) {
-                    streamer_set_output_format (&block->fmt);
-                    memcpy (&last_block_fmt, &block->fmt, sizeof (ddb_waveformat_t));
-                }
-            }
-            _format_change_wait = 0;
-            streamer_unlock ();
-        }
-
         _update_buffering_state ();
 
         if (!fileinfo_curr) {
@@ -1629,8 +1601,25 @@ streamer_thread (void *unused) {
             continue;
         }
 
-        // streamreader_read_block will lock the mutex after success
-        int res = streamreader_read_block (block, streaming_track, fileinfo_curr, mutex);
+        int res = 0;
+
+        // insert silence at the format change
+        if (memcmp (&fileinfo_curr->fmt, &prev_block_fmt, sizeof (ddb_waveformat_t))
+            && conf_format_silence > 0) {
+            memcpy (&prev_block_fmt, &fileinfo_curr->fmt, sizeof (ddb_waveformat_t));
+
+            _add_format_silence = conf_format_silence;
+        }
+        if (_add_format_silence > 0) {
+            res = streamreader_silence_block (block, streaming_track, fileinfo_curr, mutex);
+            int bytes_per_sec = fileinfo_curr->fmt.samplerate * fileinfo_curr->fmt.channels * (fileinfo_curr->fmt.bps / 8);
+            _add_format_silence -= block->size / (double)bytes_per_sec;
+        }
+        else {
+            res = streamreader_read_block (block, streaming_track, fileinfo_curr, mutex);
+        }
+
+        // streamreader has locked the mutex on success
         int last = 0;
 
         if (res >= 0) {
@@ -1642,6 +1631,8 @@ streamer_thread (void *unused) {
         if (res < 0 || last) {
             // error or eof
 
+            playItem_t *next = NULL;
+
             // handle stop after current
             int stop = 0;
             if (block->last) {
@@ -1649,9 +1640,9 @@ streamer_thread (void *unused) {
                     stop = 1;
                 }
                 else {
-                    playItem_t *next = get_next_track(playing_track, shuffle, repeat);
+                    next = get_next_track(streaming_track, shuffle, repeat);
 
-                    if (stop_after_album_check (playing_track, next)) {
+                    if (stop_after_album_check (streaming_track, next)) {
                         stop = 1;
                     }
 
@@ -1665,7 +1656,7 @@ streamer_thread (void *unused) {
                 stream_track (NULL, 0);
             }
             else {
-                streamer_next (shuffle, repeat);
+                streamer_next (shuffle, repeat, next);
             }
         }
 
@@ -1706,9 +1697,11 @@ streamer_init (void) {
     out = fopen ("out.raw", "w+b");
 #endif
     mutex = mutex_create ();
-    wdl_mutex = mutex_create ();
 
-    streamreader_init();
+    viz_init();
+
+    streamreader_init ();
+    decoded_blocks_init ();
 
     streamer_dsp_init ();
 
@@ -1733,6 +1726,7 @@ streamer_free (void) {
     thread_join (streamer_tid);
 
     streamreader_free ();
+    decoded_blocks_free ();
 
     if (first_failed_track) {
         pl_item_unref (first_failed_track);
@@ -1762,8 +1756,8 @@ streamer_free (void) {
 
     mutex_free (mutex);
     mutex = 0;
-    mutex_free (wdl_mutex);
-    wdl_mutex = 0;
+    viz_free ();
+    fft_free();
 
     streamer_dsp_chain_save();
 
@@ -1781,10 +1775,6 @@ streamer_free (void) {
     _output_buffer = NULL;
     _output_buffer_size = 0;
     _outbuffer_remaining = 0;
-
-    free (_temp_audio_buffer);
-    _temp_audio_buffer = NULL;
-    _temp_audio_buffer_size = 0;
 }
 
 static char *
@@ -1809,35 +1799,44 @@ streamer_reset (int full) { // must be called when current song changes by exter
 
     streamer_lock();
     streamreader_reset ();
+    decoded_blocks_reset();
     dsp_reset ();
     _outbuffer_remaining = 0;
     streamer_unlock();
+    viz_reset ();
 }
 
 static int
-process_output_block (streamblock_t *block, char *bytes) {
+process_output_block (streamblock_t *block, char *bytes, int bytes_available_size) {
     DB_output_t *output = plug_get_output ();
 
     if (block->pos < 0) {
         return 0;
     }
 
-    // handle change of track
-    if (block->last) {
-        update_stop_after_current ();
-    }
-    if (block->first) {
-        handle_track_change (playing_track, block->track);
-    }
-
     // A block with 0 size is a valid block, and needs to be processed as usual (code above this line).
     // But here we do early exit, because there's no data to process in it.
     if (!block->size) {
+        decoded_block_t *decoded_block = decoded_blocks_append();
+        if (decoded_block != NULL) {
+            decoded_block->track = block->track;
+            if (decoded_block->track != NULL) {
+                pl_item_ref (decoded_block->track);
+            }
+            decoded_block->last = block->last;
+            decoded_block->first = block->first;
+        }
+
         streamreader_next_block ();
         _update_buffering_state ();
         return 0;
     }
 
+    decoded_block_t *decoded_block = decoded_blocks_append();
+    if (decoded_block == NULL) {
+        return 0; // queue is full!
+    }
+
     assert (block->size > block->pos);
     int sz = block->size - block->pos;
 
@@ -1848,12 +1847,12 @@ process_output_block (streamblock_t *block, char *bytes) {
     int dspsize = 0;
     float dspratio = 1;
 
+
 #if defined(ANDROID) || defined(HAVE_XGUI)
     // android EQ and resampling require 16 bit, so convert here if needed
     int tempsize = sz * 16 / block->fmt.bps;
     int16_t *temp_audio_data = NULL;
     char *input = block->buf + block->pos;
-    block->pos += sz;
     if (block->fmt.bps != 16) {
         temp_audio_data = alloca (tempsize);
         ddb_waveformat_t out_fmt = {
@@ -1881,30 +1880,48 @@ process_output_block (streamblock_t *block, char *bytes) {
     int dsp_res = dsp_apply (&block->fmt, block->buf + block->pos, sz,
                              &datafmt, &dspbytes, &dspsize, &dspratio);
     if (dsp_res) {
-        block->pos += sz;
         sz = dspsize;
     }
     else {
         memcpy (&datafmt, &block->fmt, sizeof (ddb_waveformat_t));
         dspbytes = block->buf+block->pos;
-        block->pos += sz;
     }
 #endif
 
-    if (memcmp (&output->fmt, &datafmt, sizeof (ddb_waveformat_t))) {
+    int need_convert = memcmp (&output->fmt, &datafmt, sizeof (ddb_waveformat_t));
+    int required_size = 0;
+    if (need_convert) {
+        int input_ss = datafmt.channels * datafmt.bps/8;
+        int output_ss = output->fmt.channels * output->fmt.bps/8;
+        required_size = sz / input_ss * output_ss;
+    }
+    else {
+        required_size = sz;
+    }
+
+    // Crash here to catch the buffer issues early, instead of corrupting sound.
+    assert(bytes_available_size >= required_size);
+
+    if (need_convert) {
         sz = pcm_convert (&datafmt, dspbytes, &output->fmt, bytes, sz);
     }
     else {
         memcpy (bytes, dspbytes, sz);
     }
 
-    playpos += (float)sz/output->fmt.samplerate/((output->fmt.bps>>3)*output->fmt.channels) * dspratio;
-    playtime += (float)sz/output->fmt.samplerate/((output->fmt.bps>>3)*output->fmt.channels) * dspratio;
-
-    if (block->pos >= block->size) {
-        streamreader_next_block ();
-        _update_buffering_state ();
+    decoded_block->track = block->track;
+    if (decoded_block->track != NULL) {
+        pl_item_ref (decoded_block->track);
     }
+    decoded_block->last = block->last;
+    decoded_block->first = block->first;
+    decoded_block->total_bytes = decoded_block->remaining_bytes = sz;
+    decoded_block->is_silent_header = block->is_silent_header;
+    decoded_block->playback_time = (float)sz/output->fmt.samplerate/((output->fmt.bps>>3)*output->fmt.channels) * dspratio;
+
+    block->pos = block->size;
+    streamreader_next_block ();
+    _update_buffering_state ();
 
     return sz;
 }
@@ -1995,98 +2012,80 @@ streamer_apply_soft_volume (char *bytes, int sz) {
     }
 }
 
-static float *
-_get_temp_audio_buffer (size_t size) {
-    if (_temp_audio_buffer) {
-        if (_temp_audio_buffer_size >= size) {
-            return _temp_audio_buffer;
-        }
-        free (_temp_audio_buffer);
-    }
-    _temp_audio_buffer_size = size;
-    _temp_audio_buffer = malloc (_temp_audio_buffer_size);
-    return _temp_audio_buffer;
-}
-
-static void
-viz_process (char * restrict bytes, int bytes_size, DB_output_t *output) {
-    if (waveform_listeners || spectrum_listeners) {
-        int in_frame_size = (output->fmt.bps >> 3) * output->fmt.channels;
-        int in_frames = bytes_size / in_frame_size;
-        ddb_waveformat_t out_fmt = {
-            .bps = 32,
-            .channels = output->fmt.channels,
-            .samplerate = output->fmt.samplerate,
-            .channelmask = output->fmt.channelmask,
-            .is_float = 1,
-            .is_bigendian = 0
-        };
-
-        float *temp_audio_data = _get_temp_audio_buffer (in_frames * out_fmt.channels * sizeof (float));
-        pcm_convert (&output->fmt, bytes, &out_fmt, (char *)temp_audio_data, bytes_size);
-        ddb_audio_data_t waveform_data = {
-            .fmt = &out_fmt,
-            .data = temp_audio_data,
-            .nframes = in_frames
-        };
-        mutex_lock (wdl_mutex);
-        for (wavedata_listener_t *l = waveform_listeners; l; l = l->next) {
-            l->callback (l->ctx, &waveform_data);
-        }
-        mutex_unlock (wdl_mutex);
-
-        if (out_fmt.channels != audio_data_channels || !spectrum_listeners) {
-            audio_data_fill = 0;
-            audio_data_channels = out_fmt.channels;
-        }
-
-        if (spectrum_listeners) {
-            int remaining = in_frames;
-            do {
-                int sz = DDB_FREQ_BANDS * 2 -audio_data_fill;
-                sz = min (sz, remaining);
-                for (int c = 0; c < audio_data_channels; c++) {
-                    for (int s = 0; s < sz; s++) {
-                        audio_data[DDB_FREQ_BANDS * 2 * c + audio_data_fill + s] = temp_audio_data[(in_frames-remaining + s) * audio_data_channels + c];
-                    }
-                }
-                //            memcpy (&audio_data[audio_data_fill], &temp_audio_data[in_frames-remaining], sz * sizeof (float));
-                audio_data_fill += sz;
-                remaining -= sz;
-                if (audio_data_fill == DDB_FREQ_BANDS * 2) {
-                    for (int c = 0; c < audio_data_channels; c++) {
-                        calc_freq (&audio_data[DDB_FREQ_BANDS * 2 * c], &freq_data[DDB_FREQ_BANDS * c]);
-                    }
-                    ddb_audio_data_t spectrum_data = {
-                        .fmt = &out_fmt,
-                        .data = freq_data,
-                        .nframes = DDB_FREQ_BANDS
-                    };
-                    mutex_lock (wdl_mutex);
-                    for (wavedata_listener_t *l = spectrum_listeners; l; l = l->next) {
-                        l->callback (l->ctx, &spectrum_data);
-                    }
-                    mutex_unlock (wdl_mutex);
-                    audio_data_fill = 0;
-                }
-            } while (remaining > 0);
-        }
-    }
-}
-
-int
-streamer_read (char *bytes, int size) {
-#if 0
-    struct timeval tm1;
-    gettimeofday (&tm1, NULL);
-#endif
+static int
+_streamer_get_bytes (char *bytes, int size) {
     DB_output_t *output = plug_get_output ();
 
-    if (_format_change_wait) {
+    char *outbuffer = _get_output_buffer (OUTPUT_BUFFER_SIZE);
+
+    // consume decoded data
+    int sz = min (size, _outbuffer_remaining);
+    if (!sz) {
+        // no data available
         memset (bytes, 0, size);
         return size;
     }
 
+    // clip to frame size
+    int ss = output->fmt.channels * output->fmt.bps / 8;
+    if ((sz % ss) != 0) {
+        sz -= (sz % ss);
+    }
+
+    int rb = sz;
+    char *readptr = outbuffer;
+    char *writeptr = bytes;
+    while (rb > 0) {
+        decoded_block_t *decoded_block = decoded_blocks_current();
+        if (decoded_block == NULL) {
+            break;
+        }
+
+        // handle change of track
+        if (decoded_block->first) {
+            handle_track_change (playing_track, decoded_block->track);
+        }
+
+        if (decoded_block->remaining_bytes != 0) {
+            int got_bytes = min (rb, decoded_block->remaining_bytes);
+            memcpy (writeptr, readptr, got_bytes);
+            readptr += got_bytes;
+            writeptr += got_bytes;
+            rb -= got_bytes;
+
+            decoded_block->remaining_bytes -= got_bytes;
+        }
+
+        if (decoded_block->remaining_bytes == 0) {
+            if (decoded_block->last) {
+                update_stop_after_current ();
+            }
+
+            if (!decoded_block->is_silent_header) {
+                playpos += decoded_block->playback_time;
+                playtime += decoded_block->playback_time;
+            }
+
+            decoded_blocks_next();
+        }
+    }
+
+    sz -= rb; // how many bytes we actually got
+
+    if (sz < _outbuffer_remaining) {
+        // FIXME: This is the slowest operation on audio thread, can be optimized with a ring buffer
+        memmove (outbuffer, outbuffer + sz, _outbuffer_remaining - sz);
+    }
+    _outbuffer_remaining -= sz;
+
+    streamer_apply_soft_volume (bytes, sz);
+
+    return sz;
+}
+
+// Decode enough blocks to fill the _output_buffer, and update avg_bitrate
+static void
+_streamer_fill_playback_buffer(void) {
     streamer_lock ();
     streamblock_t *block = streamreader_get_curr_block();
     if (!block) {
@@ -2108,11 +2107,10 @@ streamer_read (char *bytes, int size) {
         streamer_unlock();
 
         if (streaming_track) {
-            memset (bytes, 0, size);
-            return size;
+            return;
         }
         _audio_stall_count++;
-        return 0;
+        return;
     }
 
     _audio_stall_count = 0;
@@ -2120,51 +2118,32 @@ streamer_read (char *bytes, int size) {
     int block_bitrate = -1;
 
     // only decode until the next format change
-    if (!memcmp (&block->fmt, &last_block_fmt, sizeof (ddb_waveformat_t))) {
-        // decode enough blocks to fill the output buffer
-        while (block != NULL && _outbuffer_remaining < size && !memcmp (&block->fmt, &last_block_fmt, sizeof (ddb_waveformat_t))) {
-            char *outbuffer = _get_output_buffer (OUTPUT_BUFFER_SIZE);
-            int rb = process_output_block (block, outbuffer + _outbuffer_remaining);
-            if (rb <= 0) {
-                break;
-            }
-            _outbuffer_remaining += rb;
-            block_bitrate = block->bitrate;
-            block = streamreader_get_curr_block();
+    // decode enough blocks to fill the output buffer
+    char *outbuffer = _get_output_buffer (OUTPUT_BUFFER_SIZE);
+    while (block != NULL
+           && decoded_blocks_have_free()
+           && decoded_blocks_playback_time_total() < conf_playback_buffer_size
+           && OUTPUT_BUFFER_SIZE-_outbuffer_remaining >= block->size * MAX_DSP_RATIO
+           && !memcmp (&block->fmt, &last_block_fmt, sizeof (ddb_waveformat_t))) {
+        int rb = process_output_block (block, outbuffer + _outbuffer_remaining, OUTPUT_BUFFER_SIZE - _outbuffer_remaining);
+        if (rb <= 0) {
+            break;
         }
+        _outbuffer_remaining += rb;
+        block_bitrate = block->bitrate;
+        block = streamreader_get_curr_block();
     }
     // empty buffer and the next block format differs? request format change!
 
     if (!_outbuffer_remaining && block && memcmp (&block->fmt, &last_block_fmt, sizeof (ddb_waveformat_t))) {
-        _format_change_wait = 1;
+        streamer_set_output_format (&block->fmt);
+        memcpy (&last_block_fmt, &block->fmt, sizeof (ddb_waveformat_t));
+
         streamer_unlock();
-        memset (bytes, 0, size);
-        return size;
+        return;
     }
     streamer_unlock ();
 
-    // consume decoded data
-    int sz = min (size, _outbuffer_remaining);
-    if (!sz) {
-        // no data available
-        memset (bytes, 0, size);
-        return size;
-    }
-
-    // clip to frame size
-    int ss = output->fmt.channels * output->fmt.bps / 8;
-    if ((sz % ss) != 0) {
-        sz -= (sz % ss);
-    }
-
-    char *outbuffer = _get_output_buffer(OUTPUT_BUFFER_SIZE);
-
-    memcpy (bytes, outbuffer, sz);
-    if (sz < _outbuffer_remaining) {
-        memmove (outbuffer, outbuffer + sz, _outbuffer_remaining - sz);
-    }
-    _outbuffer_remaining -= sz;
-
     // approximate bitrate
     if (block_bitrate != -1) {
         if (avg_bitrate == -1) {
@@ -2184,25 +2163,31 @@ streamer_read (char *bytes, int size) {
                 }
             }
         }
-//        printf ("apx bitrate: %d (last %d)\n", avg_bitrate, last_bitrate);
+        //        printf ("apx bitrate: %d (last %d)\n", avg_bitrate, last_bitrate);
     }
+}
 
-#if 0
-    struct timeval tm2;
-    gettimeofday (&tm2, NULL);
+int
+streamer_read (char *bytes, int size) {
+    DB_output_t *output = plug_get_output ();
 
-    int ms = (tm2.tv_sec*1000+tm2.tv_usec/1000) - (tm1.tv_sec*1000+tm1.tv_usec/1000);
-    printf ("streamer_read took %d ms\n", ms);
-#endif
+    // Ensure that the buffer has enough data for analysis
+    int ss = output->fmt.channels * output->fmt.bps / 8;
+    int max_bytes = output->fmt.samplerate * ss;
 
+    // Read into the output buffer
+    _streamer_fill_playback_buffer();
+
+    // Process
 #ifndef ANDROID
-
-    viz_process (bytes, sz, output);
+    // Read extra bytes from output buffer
+    int viz_bytes = min (_outbuffer_remaining, max_bytes);
+    int wave_size = size / ss;
+    viz_process (_output_buffer, viz_bytes, output, 4096, wave_size); // FIXME: fft size needs to be configurable
 #endif
 
-    streamer_apply_soft_volume (bytes, sz);
-
-    return sz;
+    // Play
+    return _streamer_get_bytes(bytes, size);
 }
 
 int
@@ -2282,6 +2267,17 @@ streamer_configchanged (void) {
     conf_streamer_samplerate_mult_48 = new_conf_streamer_samplerate_mult_48;
     conf_streamer_samplerate_mult_44 = new_conf_streamer_samplerate_mult_44;
 
+    conf_format_silence = conf_get_float ("streamer.format_change_silence", -1.f);
+
+    int playback_buffer_size = conf_get_int ("streamer.playback_buffer_size", 300);
+    if (playback_buffer_size < 100) {
+        playback_buffer_size = 100;
+    }
+    else if (playback_buffer_size > 2000) {
+        playback_buffer_size = 2000;
+    }
+    conf_playback_buffer_size = playback_buffer_size / 1000.f;
+
     streamer_unlock ();
 
     streamreader_configchanged ();
@@ -2345,6 +2341,23 @@ _play_track (playItem_t *it, int startpaused) {
     }
 }
 
+static void
+_rebuild_shuffle_albums_after_manual_trigger(playlist_t *plt, playItem_t *it) {
+    plt_reshuffle (plt, NULL, NULL);
+
+    ddb_shuffle_t shuffle = streamer_get_shuffle ();
+    if (shuffle == DDB_SHUFFLE_ALBUMS) {
+        pl_lock ();
+        _streamer_mark_album_played_up_to (it);
+        pl_unlock ();
+    }
+    else {
+        // This ensures that the manually triggered item becomes first in shuffle queue.
+        // It works because shufflerating is generated using rand(), which gives only numbers in the [0..RAND_MAX] range.
+        it->shufflerating = -1;
+    }
+}
+
 // play track in current playlist by index;
 // negative index will stop playback
 static void
@@ -2372,28 +2385,7 @@ play_index (int idx, int startpaused) {
     pl_unlock();
 
     // rebuild shuffle order
-    plt_reshuffle (plt, NULL, NULL);
-
-    ddb_shuffle_t shuffle = streamer_get_shuffle ();
-    if (shuffle == DDB_SHUFFLE_ALBUMS) {
-        pl_lock ();
-        _streamer_mark_album_played_up_to (it);
-
-        // mark songs from all other albums as played
-        playItem_t *i = plt->head[PL_MAIN];
-        while (i) {
-            if (i->shufflerating != it->shufflerating) {
-                i->played = 1;
-            }
-            i = i->next[PL_MAIN];
-        }
-        pl_unlock ();
-    }
-    else {
-        // This ensures that the manually triggered item becomes first in shuffle queue.
-        // It works because shufflerating is generated using rand(), which gives only numbers in the [0..RAND_MAX] range.
-        it->shufflerating = -1;
-    }
+    _rebuild_shuffle_albums_after_manual_trigger (plt, it);
 
     _play_track(it, startpaused);
 
@@ -2404,6 +2396,8 @@ play_index (int idx, int startpaused) {
 error:
     output->stop ();
     streamer_reset (1);
+    viz_reset();
+    viz_process(NULL, 0, output, 0, 0);
 
     streamer_lock();
     _handle_playback_stopped ();
@@ -2530,6 +2524,10 @@ play_next (int dir, ddb_shuffle_t shuffle, ddb_repeat_t repeat) {
         return;
     }
 
+    if (dir == 0) {
+        // rebuild shuffle order
+        _rebuild_shuffle_albums_after_manual_trigger (streamer_playlist, next);
+    }
     _play_track(next, 0);
     pl_item_unref(next);
 }
@@ -2602,6 +2600,7 @@ _streamer_mark_album_played_up_to (playItem_t *item) {
     pl_lock ();
     const char *alb = pl_find_meta_raw (item, "album");
     const char *art = pl_find_meta_raw (item, "artist");
+    item->played = 1;
     playItem_t *next = item->prev[PL_MAIN];
     while (next) {
         if (alb == pl_find_meta_raw (next, "album") && art == pl_find_meta_raw (next, "artist")) {
@@ -2630,68 +2629,6 @@ streamer_shuffle_changed (ddb_shuffle_t prev, ddb_shuffle_t shuffle) {
     }
 }
 
-void
-vis_waveform_listen (void *ctx, void (*callback)(void *ctx, ddb_audio_data_t *data)) {
-    mutex_lock (wdl_mutex);
-    wavedata_listener_t *l = malloc (sizeof (wavedata_listener_t));
-    memset (l, 0, sizeof (wavedata_listener_t));
-    l->ctx = ctx;
-    l->callback = callback;
-    l->next = waveform_listeners;
-    waveform_listeners = l;
-    mutex_unlock (wdl_mutex);
-}
-
-void
-vis_waveform_unlisten (void *ctx) {
-    mutex_lock (wdl_mutex);
-    wavedata_listener_t *l, *prev = NULL;
-    for (l = waveform_listeners; l; prev = l, l = l->next) {
-        if (l->ctx == ctx) {
-            if (prev) {
-                prev->next = l->next;
-            }
-            else {
-                waveform_listeners = l->next;
-            }
-            free (l);
-            break;
-        }
-    }
-    mutex_unlock (wdl_mutex);
-}
-
-void
-vis_spectrum_listen (void *ctx, void (*callback)(void *ctx, ddb_audio_data_t *data)) {
-    mutex_lock (wdl_mutex);
-    wavedata_listener_t *l = malloc (sizeof (wavedata_listener_t));
-    memset (l, 0, sizeof (wavedata_listener_t));
-    l->ctx = ctx;
-    l->callback = callback;
-    l->next = spectrum_listeners;
-    spectrum_listeners = l;
-    mutex_unlock (wdl_mutex);
-}
-
-void
-vis_spectrum_unlisten (void *ctx) {
-    mutex_lock (wdl_mutex);
-    wavedata_listener_t *l, *prev = NULL;
-    for (l = spectrum_listeners; l; prev = l, l = l->next) {
-        if (l->ctx == ctx) {
-            if (prev) {
-                prev->next = l->next;
-            }
-            else {
-                spectrum_listeners = l->next;
-            }
-            free (l);
-            break;
-        }
-    }
-    mutex_unlock (wdl_mutex);
-}
-
 void
 streamer_set_streamer_playlist (playlist_t *plt) {
     pl_lock ();
