@@ -83,6 +83,9 @@ streamreader_get_next_block (void) {
     if (block_next->pos >= 0) {
         return NULL; // all buffers full
     }
+
+    // FIXME: initialize
+    block_next->is_silent_header = 0;
     return block_next;
 }
 
@@ -94,13 +97,16 @@ streamreader_configchanged (void) {
 int
 streamreader_read_block (streamblock_t *block, playItem_t *track, DB_fileinfo_t *fileinfo, uint64_t mutex) {
     int size = BLOCK_SIZE;
-    if (!fileinfo->plugin) {
+    int samplesize = fileinfo->fmt.channels * (fileinfo->fmt.bps>>3);
+
+    // NOTE: samplesize has to be checked to protect against faulty input plugins
+
+    if (!fileinfo->plugin || samplesize <= 0) {
         // return dummy block for a failed track
         _firstblock = 1;
         size = 0;
     }
     else {
-        int samplesize = fileinfo->fmt.channels * (fileinfo->fmt.bps>>3);
         // clip size to max possible, with current sample format
         int mod = size % samplesize;
         if (mod) {
@@ -143,6 +149,9 @@ streamreader_read_block (streamblock_t *block, playItem_t *track, DB_fileinfo_t
     }
     memcpy (&block->fmt, &fileinfo->fmt, sizeof (ddb_waveformat_t));
     block->track = track;
+    if (block->track != NULL) {
+        pl_item_ref(block->track);
+    }
 
     if (size > 0) {
         int input_does_rg = fileinfo->plugin->plugin.flags & DDB_PLUGIN_FLAG_REPLAYGAIN;
@@ -170,6 +179,35 @@ streamreader_read_block (streamblock_t *block, playItem_t *track, DB_fileinfo_t
     return 0;
 }
 
+int
+streamreader_silence_block (streamblock_t *block, playItem_t *track, DB_fileinfo_t *fileinfo, uint64_t mutex) {
+    curr_block_bitrate = -1;
+    mutex_lock (mutex);
+    block->bitrate = -1;
+    block->pos = 0;
+    memset (block->buf, 0, BLOCK_SIZE);
+    block->size = BLOCK_SIZE;
+
+    memcpy (&block->fmt, &fileinfo->fmt, sizeof (ddb_waveformat_t));
+    block->track = track;
+    if (block->track != NULL) {
+        pl_item_ref(block->track);
+    }
+    block->is_silent_header = 1;
+
+    if (_firstblock) {
+        block->first = 1;
+        _firstblock = 0;
+    }
+    else {
+        block->first = 0;
+    }
+
+    block->last = 0;
+
+    return 0;
+}
+
 void
 streamreader_enqueue_block (streamblock_t *block) {
     // block is passed just for sanity checking
@@ -200,6 +238,9 @@ streamreader_get_curr_block (void) {
 static void
 _streamreader_release_block (streamblock_t *block) {
     block->pos = -1;
+    if (block->track != NULL) {
+        pl_item_unref(block->track);
+    }
     block->track = NULL;
     block->queued = 0;
 
@@ -231,7 +272,8 @@ streamreader_reset (void) {
     streamblock_t *b = blocks;
     while (b) {
         b->pos = -1;
-        if (b->track) {
+        if (b->track != NULL) {
+            pl_item_unref(b->track);
             b->track = NULL;
         }
         b->queued = 0;
